@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001 by Solar Designer
+ * Copyright (c) 1996-2001,2012 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 #include <string.h>
@@ -12,8 +17,8 @@
 #include "common.h"
 #include "formats.h"
 
-#define FORMAT_LABEL			"afs"
-#define FORMAT_NAME			"Kerberos AFS DES"
+#define FORMAT_LABEL			"AFS"
+#define FORMAT_NAME			"Kerberos AFS"
 
 #define BENCHMARK_COMMENT		""
 #define BENCHMARK_LENGTH		8
@@ -36,7 +41,9 @@ static struct fmt_tests tests[] = {
 #define ALGORITHM_NAME			DES_STD_ALGORITHM_NAME
 
 #define BINARY_SIZE			(3 * ARCH_SIZE)
+#define BINARY_ALIGN			ARCH_SIZE
 #define SALT_SIZE			40
+#define SALT_ALIGN			1
 
 #define MIN_KEYS_PER_CRYPT		0x80
 #define MAX_KEYS_PER_CRYPT		0x100
@@ -90,7 +97,7 @@ static union {
 } AFS_long_KS;
 static DES_binary AFS_long_IV_binary;
 
-static void init(void)
+static void init(struct fmt_main *self)
 {
 	ARCH_WORD_32 block[2];
 #if !ARCH_LITTLE_ENDIAN
@@ -113,7 +120,7 @@ static void init(void)
 	memcpy(AFS_long_IV_binary, DES_IV, sizeof(DES_binary));
 }
 
-static int valid(char *ciphertext)
+static int valid(char *ciphertext, struct fmt_main *self)
 {
 	char *pos;
 	int index, count;
@@ -279,8 +286,9 @@ static char *get_key(int index)
 	return buffer[index].key;
 }
 
-static void crypt_all(int count)
+static int crypt_all(int *pcount, struct db_salt *salt)
 {
+	int count = *pcount;
 	int index, pos, length;
 	char xor[8];
 	ARCH_WORD_32 space[(PLAINTEXT_LENGTH + SALT_SIZE + 8) / 4 + 1];
@@ -376,6 +384,8 @@ static void crypt_all(int count)
 		buffer[index].aligned.binary[0] = block[0] | 0x01010101;
 		buffer[index].aligned.binary[1] = block[1] | 0x01010101;
 	}
+
+	return count;
 }
 
 static int cmp_all(void *binary, int count)
@@ -438,22 +448,30 @@ struct fmt_main fmt_AFS = {
 		BENCHMARK_LENGTH,
 		PLAINTEXT_LENGTH,
 		BINARY_SIZE,
+		BINARY_ALIGN,
 		SALT_SIZE,
+		SALT_ALIGN,
 		MIN_KEYS_PER_CRYPT,
 		MAX_KEYS_PER_CRYPT,
 		FMT_CASE | FMT_8_BIT,
 		tests
 	}, {
 		init,
+		fmt_default_done,
+		fmt_default_reset,
+		fmt_default_prepare,
 		valid,
 		fmt_default_split,
 		get_binary,
 		salt,
+		fmt_default_source,
 		{
 			binary_hash_0,
 			binary_hash_1,
 			binary_hash_2,
 			NULL,
+			NULL,
+			NULL,
 			NULL
 		},
 		fmt_default_salt_hash,
@@ -467,6 +485,8 @@ struct fmt_main fmt_AFS = {
 			get_hash_1,
 			get_hash_2,
 			NULL,
+			NULL,
+			NULL,
 			NULL
 		},
 		cmp_all,
