@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001,2008,2010,2011 by Solar Designer
+ * Copyright (c) 1996-2001,2008,2010-2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 #include <stdlib.h>
@@ -12,17 +17,19 @@
 #include "common.h"
 #include "formats.h"
 
-#define FORMAT_LABEL			"bf"
-#define FORMAT_NAME			"OpenBSD Blowfish"
+#define FORMAT_LABEL			"bcrypt"
+#define FORMAT_NAME			""
 
-#define BENCHMARK_COMMENT		" (x32)"
+#define BENCHMARK_COMMENT		" (\"$2a$05\", 32 iterations)"
 #define BENCHMARK_LENGTH		-1
 
 #define PLAINTEXT_LENGTH		72
 #define CIPHERTEXT_LENGTH		60
 
 #define BINARY_SIZE			4
+#define BINARY_ALIGN			4
 #define SALT_SIZE			sizeof(BF_salt)
+#define SALT_ALIGN			4
 
 #define MIN_KEYS_PER_CRYPT		BF_Nmin
 #define MAX_KEYS_PER_CRYPT		BF_N
@@ -42,7 +49,7 @@ static struct fmt_tests tests[] = {
 		"chars after 72 are ignored"},
 	{"$2x$05$/OK.fbVrR/bpIqNJ5ianF.CE5elHaaO4EbggVDjb8P19RukzXSM3e",
 		"\xa3"},
-	{"$2a$05$/OK.fbVrR/bpIqNJ5ianF.Sa7shbm4.OzKpvFnX1pQLmQW96oUlCq",
+	{"$2y$05$/OK.fbVrR/bpIqNJ5ianF.Sa7shbm4.OzKpvFnX1pQLmQW96oUlCq",
 		"\xa3"},
 	{"$2x$05$6bNw2HLQYeqHYyBfLMsv/OiwqTymGIGzFsA4hOTWebfehXHNprcAS",
 		"\xd1\x91"},
@@ -86,40 +93,41 @@ static BF_salt saved_salt;
 struct fmt_main fmt_BF;
 #endif
 
-static void init(void)
+static void init(struct fmt_main *self)
 {
 #ifdef _OPENMP
-	int n = BF_Nmin * omp_get_max_threads();
+	int n = BF_Nmin * omp_get_max_threads(), max;
 	if (n < BF_Nmin)
 		n = BF_Nmin;
 	if (n > BF_N)
 		n = BF_N;
 	fmt_BF.params.min_keys_per_crypt = n;
-	n *= BF_cpt;
-	if (n > BF_N)
-		n = BF_N;
-	fmt_BF.params.max_keys_per_crypt = n;
+	max = n * BF_cpt;
+	while (max > BF_N)
+		max -= n;
+	fmt_BF.params.max_keys_per_crypt = max;
 #endif
 
-	keys_mode = 'a';
+	keys_mode = 'y';
 	sign_extension_bug = 0;
 }
 
-static int valid(char *ciphertext)
+static int valid(char *ciphertext, struct fmt_main *self)
 {
 	int rounds;
 	char *pos;
 
 	if (strncmp(ciphertext, "$2a$", 4) &&
-	    strncmp(ciphertext, "$2x$", 4)) return 0;
+	    strncmp(ciphertext, "$2x$", 4) &&
+	    strncmp(ciphertext, "$2y$", 4))
+		return 0;
 
 	if (ciphertext[4] < '0' || ciphertext[4] > '9') return 0;
 	if (ciphertext[5] < '0' || ciphertext[5] > '9') return 0;
+	if (ciphertext[6] != '$') return 0;
 	rounds = atoi(ciphertext + 4);
 	if (rounds < 4 || rounds > 31) return 0;
 
-	if (ciphertext[6] != '$') return 0;
-
 	for (pos = &ciphertext[7]; atoi64[ARCH_INDEX(*pos)] != 0x7F; pos++);
 	if (*pos || pos - ciphertext != CIPHERTEXT_LENGTH) return 0;
 
@@ -154,6 +162,16 @@ static int binary_hash_4(void *binary)
 	return *(BF_word *)binary & 0xFFFFF;
 }
 
+static int binary_hash_5(void *binary)
+{
+	return *(BF_word *)binary & 0xFFFFFF;
+}
+
+static int binary_hash_6(void *binary)
+{
+	return *(BF_word *)binary & 0x7FFFFFF;
+}
+
 static int get_hash_0(int index)
 {
 	return BF_out[index][0] & 0xF;
@@ -179,9 +197,19 @@ static int get_hash_4(int index)
 	return BF_out[index][0] & 0xFFFFF;
 }
 
+static int get_hash_5(int index)
+{
+	return BF_out[index][0] & 0xFFFFFF;
+}
+
+static int get_hash_6(int index)
+{
+	return BF_out[index][0] & 0x7FFFFFF;
+}
+
 static int salt_hash(void *salt)
 {
-	return ((BF_salt *)salt)->salt[0] & 0x3FF;
+	return ((BF_salt *)salt)->salt[0] & (SALT_HASH_SIZE - 1);
 }
 
 static void set_salt(void *salt)
@@ -201,8 +229,10 @@ static char *get_key(int index)
 	return saved_key[index];
 }
 
-static void crypt_all(int count)
+static int crypt_all(int *pcount, struct db_salt *salt)
 {
+	int count = *pcount;
+
 	if (keys_mode != saved_salt.subtype) {
 		int i;
 
@@ -213,6 +243,8 @@ static void crypt_all(int count)
 	}
 
 	BF_std_crypt(&saved_salt, count);
+
+	return count;
 }
 
 static int cmp_all(void *binary, int count)
@@ -256,23 +288,34 @@ struct fmt_main fmt_BF = {
 		BENCHMARK_LENGTH,
 		PLAINTEXT_LENGTH,
 		BINARY_SIZE,
+		BINARY_ALIGN,
 		SALT_SIZE,
+		SALT_ALIGN,
 		MIN_KEYS_PER_CRYPT,
 		MAX_KEYS_PER_CRYPT,
-		FMT_CASE | FMT_8_BIT | FMT_OMP,
+#if BF_mt > 1
+		FMT_OMP |
+#endif
+		FMT_CASE | FMT_8_BIT,
 		tests
 	}, {
 		init,
+		fmt_default_done,
+		fmt_default_reset,
+		fmt_default_prepare,
 		valid,
 		fmt_default_split,
 		BF_std_get_binary,
 		BF_std_get_salt,
+		fmt_default_source,
 		{
 			binary_hash_0,
 			binary_hash_1,
 			binary_hash_2,
 			binary_hash_3,
-			binary_hash_4
+			binary_hash_4,
+			binary_hash_5,
+			binary_hash_6
 		},
 		salt_hash,
 		set_salt,
@@ -285,7 +328,9 @@ struct fmt_main fmt_BF = {
 			get_hash_1,
 			get_hash_2,
 			get_hash_3,
-			get_hash_4
+			get_hash_4,
+			get_hash_5,
+			get_hash_6
 		},
 		cmp_all,
 		cmp_one,
