@@ -2,6 +2,11 @@
  * This file is part of John the Ripper password cracker,
  * Copyright (c) 1996-2001,2008,2010,2011 by Solar Designer
  *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
+ *
  * A public domain version of this code, with reentrant and crypt(3)
  * interfaces added, but optimizations specific to password cracking
  * removed, is available at:
@@ -817,7 +822,10 @@ void *BF_std_get_salt(char *ciphertext)
 	BF_swap(salt.salt, 4);
 
 	salt.rounds = atoi(&ciphertext[4]);
-	salt.subtype = ciphertext[2];
+	if (ciphertext[2] == 'a')
+		salt.subtype = 'y';
+	else
+		salt.subtype = ciphertext[2];
 
 	return &salt;
 }
