@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
  * Copyright (c) 1996-2001,2008,2010,2011 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -37,7 +42,7 @@ typedef BF_word BF_binary[6];
 
 #if defined(_OPENMP) && !BF_ASM
 #define BF_cpt				3
-#define BF_mt				96
+#define BF_mt				192
 #define BF_N				(BF_Nmin * BF_mt)
 #else
 #define BF_mt				1
@@ -55,9 +60,9 @@ extern BF_binary BF_out[BF_N];
 extern unsigned char BF_atoi64[0x80];
 
 #if BF_X2
-#define BF_ALGORITHM_NAME		"32/" ARCH_BITS_STR " X2"
+#define BF_ALGORITHM_NAME		"Blowfish 32/" ARCH_BITS_STR " X2"
 #else
-#define BF_ALGORITHM_NAME		"32/" ARCH_BITS_STR
+#define BF_ALGORITHM_NAME		"Blowfish 32/" ARCH_BITS_STR
 #endif
 
 /*
