@@ -2,6 +2,11 @@
  * This file is part of John the Ripper password cracker,
  * Copyright (c) 1996-2001,2003,2006,2011 by Solar Designer
  *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
+ *
  * This implementation of FreeBSD-style MD5-based crypt(3) password hashing
  * supports passwords of up to 15 characters long only since this lets us use a
  * significantly faster algorithm. -- SD
@@ -13,7 +18,16 @@
 #include "common.h"
 #include "MD5_std.h"
 
+#if MD5_std_mt
+#include <omp.h>
+int MD5_std_min_kpc, MD5_std_max_kpc;
+int MD5_std_nt;
+MD5_std_combined *MD5_std_all_p = NULL;
+static char saved_salt[9];
+static int salt_changed;
+#else
 MD5_std_combined CC_CACHE_ALIGN MD5_std_all;
+#endif
 
 #if !MD5_IMM
 static MD5_data MD5_data_init = {
@@ -265,17 +279,6 @@ static MD5_data MD5_data_init = {
 	ROTATE_LEFT ((a), (s)); \
 	(a) += (b);
 
-#if MD5_X2
-static void MD5_body(MD5_word x0[15], MD5_word x1[15],
-	MD5_word out0[4], MD5_word out1[4]);
-#else
-static void MD5_body(MD5_word x[15], MD5_word out[4]);
-#endif
-
-#else
-
-extern void MD5_body(MD5_word x[15], MD5_word out[4]);
-
 #endif
 
 static unsigned char PADDING[56] = {
@@ -286,20 +289,23 @@ static unsigned char PADDING[56] = {
 
 #if ARCH_LITTLE_ENDIAN
 
-#define MD5_swap(x, y, count)
+#define MD5_swap(dst, src, count)
 
 #else
 
-static void MD5_swap(MD5_word *x, MD5_word *y, int count)
-{
-	MD5_word tmp, mask;
-
-	mask = OOFFOOFF;
-	do {
-		tmp = *x++;
-		ROTATE_LEFT(tmp, 16);
-		*y++ = ((tmp & mask) << 8) | ((tmp >> 8) & mask);
-	} while (--count);
+#define MD5_swap(dst, src, count) \
+{ \
+	MD5_word *dptr = (dst), *sptr = (src); \
+	int loop_count = (count); \
+	MD5_word mask = OOFFOOFF; \
+	do { \
+		MD5_word tmp = *sptr++; \
+		ROTATE_LEFT(tmp, 16); \
+		*dptr++ = ((tmp & mask) << 8) | ((tmp >> 8) & mask); \
+		tmp = *sptr++; \
+		ROTATE_LEFT(tmp, 16); \
+		*dptr++ = ((tmp & mask) << 8) | ((tmp >> 8) & mask); \
+	} while ((loop_count -= 2)); \
 }
 
 #endif
@@ -310,47 +316,80 @@ static void MD5_swap(MD5_word *x, MD5_word *y, int count)
 #define prefix				MD5_std_all.prefix
 #define prelen				MD5_std_all.prelen
 
-static void init_line(int line, int index, MD5_block *even, MD5_block *odd)
-{
-	order[line][index].even = even;
-	order[line][index].odd = odd;
-}
-
 void MD5_std_init(void)
 {
 	int index;
 	MD5_pool *current;
+#if MD5_std_mt
+	int t, n;
 
-#if !MD5_IMM
-	MD5_std_all.data = MD5_data_init;
+	if (!MD5_std_all_p) {
+		n = omp_get_max_threads();
+		if (n < 1)
+			n = 1;
+		if (n > MD5_std_mt_max)
+			n = MD5_std_mt_max;
+		MD5_std_min_kpc = n * MD5_N;
+		{
+			int max = n * MD5_std_cpt;
+			while (max > MD5_std_mt_max)
+				max -= n;
+			n = max;
+		}
+		MD5_std_max_kpc = n * MD5_N;
+/*
+ * The array of MD5_std_all's is not exactly tiny, but we use mem_alloc_tiny()
+ * for its alignment support and error checking.  We do not need to free() this
+ * memory anyway.
+ */
+		MD5_std_all_p = mem_alloc_tiny(n * MD5_std_all_size,
+		    MEM_ALIGN_PAGE);
+		MD5_std_nt = n;
+	}
 #endif
 
-	for (index = 0, current = pool; index < MD5_N; index++, current++) {
-		init_line(0, index, &current->e.p, &current->o.psp);
-		init_line(1, index, &current->e.spp, &current->o.pp);
-		init_line(2, index, &current->e.spp, &current->o.psp);
-		init_line(3, index, &current->e.pp, &current->o.ps);
-		init_line(4, index, &current->e.spp, &current->o.pp);
-		init_line(5, index, &current->e.spp, &current->o.psp);
-		init_line(6, index, &current->e.pp, &current->o.psp);
-		init_line(7, index, &current->e.sp, &current->o.pp);
-		init_line(8, index, &current->e.spp, &current->o.psp);
-		init_line(9, index, &current->e.pp, &current->o.psp);
-		init_line(10, index, &current->e.spp, &current->o.p);
-		init_line(11, index, &current->e.spp, &current->o.psp);
-		init_line(12, index, &current->e.pp, &current->o.psp);
-		init_line(13, index, &current->e.spp, &current->o.pp);
-		init_line(14, index, &current->e.sp, &current->o.psp);
-		init_line(15, index, &current->e.pp, &current->o.psp);
-		init_line(16, index, &current->e.spp, &current->o.pp);
-		init_line(17, index, &current->e.spp, &current->o.ps);
-		init_line(18, index, &current->e.pp, &current->o.psp);
-		init_line(19, index, &current->e.spp, &current->o.pp);
-		init_line(20, index, &current->e.spp, &current->o.psp);
+	for_each_t(MD5_std_nt) {
+#if !MD5_IMM
+		MD5_std_all.data = MD5_data_init;
+#endif
+
+		current = pool;
+		for (index = 0; index < MD5_N; index++) {
+#define init_line(line, init_even, init_odd) \
+	order[line][index].even = init_even; \
+	order[line][index].odd = init_odd;
+			init_line(0, &current->e.p, &current->o.psp);
+			init_line(1, &current->e.spp, &current->o.pp);
+			init_line(2, &current->e.spp, &current->o.psp);
+			init_line(3, &current->e.pp, &current->o.ps);
+			init_line(4, &current->e.spp, &current->o.pp);
+			init_line(5, &current->e.spp, &current->o.psp);
+			init_line(6, &current->e.pp, &current->o.psp);
+			init_line(7, &current->e.sp, &current->o.pp);
+			init_line(8, &current->e.spp, &current->o.psp);
+			init_line(9, &current->e.pp, &current->o.psp);
+			init_line(10, &current->e.spp, &current->o.p);
+			init_line(11, &current->e.spp, &current->o.psp);
+			init_line(12, &current->e.pp, &current->o.psp);
+			init_line(13, &current->e.spp, &current->o.pp);
+			init_line(14, &current->e.sp, &current->o.psp);
+			init_line(15, &current->e.pp, &current->o.psp);
+			init_line(16, &current->e.spp, &current->o.pp);
+			init_line(17, &current->e.spp, &current->o.ps);
+			init_line(18, &current->e.pp, &current->o.psp);
+			init_line(19, &current->e.spp, &current->o.pp);
+			init_line(20, &current->e.spp, &current->o.psp);
+#undef init_line
+			current++;
+		}
 	}
 }
 
+#if MD5_std_mt
+static MAYBE_INLINE void MD5_std_set_salt_for_thread(int t, char *salt)
+#else
 void MD5_std_set_salt(char *salt)
+#endif
 {
 	int length;
 
@@ -370,11 +409,21 @@ void MD5_std_set_salt(char *salt)
 	}
 }
 
+#if MD5_std_mt
+void MD5_std_set_salt(char *salt)
+{
+	memcpy(saved_salt, salt, sizeof(saved_salt));
+	salt_changed = 1;
+}
+#endif
+
 void MD5_std_set_key(char *key, int index)
 {
 	int length;
 	MD5_pool *current;
 
+	init_t();
+
 	for (length = 0; key[length] && length < 15; length++);
 	current = &pool[index];
 
@@ -409,193 +458,30 @@ void MD5_std_set_key(char *key, int index)
 	order[19][index].length = current->l.pp;
 }
 
-void MD5_std_crypt(void)
-{
-	int length, index, mask;
-	MD5_pattern *line;
-#if ARCH_LITTLE_ENDIAN
-	MD5_word *last0;
-#endif
-#if MD5_X2
-	MD5_pool *key;
-#if ARCH_LITTLE_ENDIAN
-	MD5_word *last1;
-#endif
-#endif
-
-#if MD5_X2
-	for (index = 0, key = pool; index < MD5_N; index++, key++) {
+#if MD5_ASM
+extern void MD5_body(MD5_word x[15], MD5_word out[4]);
 #else
-#define index	0
-#define key	pool
-#endif
-		memcpy(key->o.ps.b, key->o.p.b, key->l.p);
-		memcpy(&key->o.ps.b[key->l.p], key->s, key->l.s);
-		key->l.ps = key->l.p + key->l.s;
-		memcpy(&key->o.ps.b[key->l.ps + 16], PADDING,
-			40 - key->l.ps);
-		key->o.ps.w[14] = (key->l.ps + 16) << 3;
 
-		memcpy(key->o.psp.b, key->o.ps.b, key->l.ps);
-		memcpy(&key->o.psp.b[key->l.ps], key->o.p.b, key->l.p);
-		key->l.psp = key->l.ps + key->l.p;
-		memcpy(&key->o.psp.b[key->l.psp + 16], PADDING,
-			40 - key->l.psp);
-		key->o.psp.w[14] = (key->l.psp + 16) << 3;
-
-		memcpy(&key->e.sp.b[16], key->s, key->l.s);
-		memcpy(&key->e.sp.b[16 + key->l.s], key->o.p.b,
-			key->l.p);
-		memcpy(&key->e.sp.b[16 + key->l.ps], PADDING,
-			40 - key->l.ps);
-		key->e.sp.w[14] = (key->l.ps + 16) << 3;
-		MD5_swap(key->e.sp.w, key->e.sp.w, 14);
-
-		memcpy(&key->e.spp.b[16], key->s, key->l.s);
-		memcpy(&key->e.spp.b[16 + key->l.s], key->o.pp.b,
-			key->l.pp);
-		memcpy(&key->e.spp.b[16 + key->l.psp], PADDING,
-			40 - key->l.psp);
-		key->e.spp.w[14] = (key->l.psp + 16) << 3;
-		MD5_swap(key->e.spp.w, key->e.spp.w, 14);
-
-		order[0][index].length = key->l.psp;
-		order[2][index].length = key->l.psp;
-		order[3][index].length = key->l.ps;
-		order[5][index].length = key->l.psp;
-		order[6][index].length = key->l.psp;
-		order[8][index].length = key->l.psp;
-		order[9][index].length = key->l.psp;
-		order[11][index].length = key->l.psp;
-		order[12][index].length = key->l.psp;
-		order[14][index].length = key->l.psp;
-		order[15][index].length = key->l.psp;
-		order[17][index].length = key->l.ps;
-		order[18][index].length = key->l.psp;
-		order[20][index].length = key->l.psp;
-
-		memcpy(&block[index], key->o.psp.b, key->l.psp);
-		memcpy(&block[index].b[key->l.psp], PADDING, 56 - key->l.psp);
-		block[index].w[14] = key->l.psp << 3;
-		MD5_swap(block[index].w, block[index].w, 14);
-#if MD5_X2
-	}
-
-	MD5_body(block[0].w, block[1].w, MD5_out[0], MD5_out[1]);
-	MD5_swap(MD5_out[0], MD5_out[0], 8);
+/*
+ * x86-64 implies a fairly recent CPU, so presumably its L1 instruction cache
+ * is large enough.
+ */
+#ifdef __x86_64__
+#define MAYBE_INLINE_BODY MAYBE_INLINE
 #else
-	MD5_body(block[0].w, MD5_out[0]);
-	MD5_swap(MD5_out[0], MD5_out[0], 4);
+#define MAYBE_INLINE_BODY
 #endif
 
-#if MD5_X2
-	for (index = 0, key = pool; index < MD5_N; index++, key++) {
-#endif
-		memcpy(&block[index], key->o.p.b, key->l.p);
-		memcpy(&block[index].b[key->l.p], prefix, prelen);
-		memcpy(&block[index].b[key->l.p + prelen], key->s, key->l.s);
-		memcpy(&block[index].b[key->l.ps + prelen],
-			MD5_out[index], key->l.p);
-		length = key->l.psp + prelen;
-		if ((mask = key->l.p))
-		do {
-			block[index].b[length++] =
-				(mask & 1) ? 0 : key->o.p.b[0];
-		} while (mask >>= 1);
-		memcpy(&block[index].b[length], PADDING, 56 - length);
-		block[index].w[14] = length << 3;
-		MD5_swap(block[index].w, block[index].w, 14);
-#if MD5_X2
-	}
-#else
-#undef index
-#undef key
-#endif
-
-#if MD5_X2
-	MD5_body(block[0].w, block[1].w,
-		order[0][0].even->w, order[0][1].even->w);
-#else
-	MD5_body(block[0].w, order[0][0].even->w);
-#endif
-
-	index = 500; line = order[0];
-	do {
-#if ARCH_LITTLE_ENDIAN
-#if ARCH_ALLOWS_UNALIGNED
-#if MD5_X2
-		MD5_body(line[0].even->w, line[1].even->w,
-			(MD5_word *)&line[0].odd->b[line[0].length],
-			(MD5_word *)&line[1].odd->b[line[1].length]);
-#else
-		MD5_body(line[0].even->w,
-			(MD5_word *)&line[0].odd->b[line[0].length]);
-#endif
-#else
-#if MD5_X2
-		MD5_body(line[0].even->w, line[1].even->w,
-			MD5_out[0], MD5_out[1]);
-		memcpy(&line[0].odd->b[line[0].length], MD5_out[0], 16);
-		memcpy(&line[1].odd->b[line[1].length], MD5_out[1], 16);
-#else
-		if (((ARCH_WORD)&line[0].odd->b[line[0].length]) & 3) {
-			MD5_body(line[0].even->w, MD5_out[0]);
-			memcpy(&line[0].odd->b[line[0].length],
-				MD5_out[0], 16);
-		} else {
-			MD5_body(line[0].even->w,
-				(MD5_word *)&line[0].odd->b[line[0].length]);
-		}
-#endif
-#endif
-		last0 = line[0].odd->w;
-#if MD5_X2
-		last1 = line[1].odd->w;
-		if ((line += 2) > &order[20][MD5_N - 1]) line = order[0];
-		MD5_body(last0, last1, line[0].even->w, line[1].even->w);
-#else
-		if (++line > &order[20][0]) line = order[0];
-		MD5_body(last0, line[0].even->w);
-#endif
-#else
-#if MD5_X2
-		MD5_body(line[0].even->w, line[1].even->w,
-			MD5_out[0], MD5_out[1]);
-		MD5_swap(MD5_out[0], MD5_out[0], 8);
-#else
-		MD5_body(line[0].even->w, MD5_out[0]);
-		MD5_swap(MD5_out[0], MD5_out[0], 4);
-#endif
-		memcpy(&line[0].odd->b[line[0].length], MD5_out[0], 16);
-#if MD5_X2
-		memcpy(&line[1].odd->b[line[1].length], MD5_out[1], 16);
-#endif
-		MD5_swap(line[0].odd->w, block[0].w, 14);
-		block[0].w[14] = line[0].odd->w[14];
-#if MD5_X2
-		MD5_swap(line[1].odd->w, block[1].w, 14);
-		block[1].w[14] = line[1].odd->w[14];
-		if ((line += 2) > &order[20][MD5_N - 1]) line = order[0];
-		MD5_body(block[0].w, block[1].w,
-			line[0].even->w, line[1].even->w);
-#else
-		if (++line > &order[20][0]) line = order[0];
-		MD5_body(block[0].w, line[0].even->w);
-#endif
-#endif
-	} while (--index);
-
-	memcpy(MD5_out[0], line[0].even, 16);
-#if MD5_X2
-	memcpy(MD5_out[1], line[1].even, 16);
-#endif
-}
-
-#if !MD5_ASM
-
 #if !MD5_X2
 
-static void MD5_body(MD5_word x[15], MD5_word out[4])
+#if MD5_std_mt
+#define MD5_body(x, out) \
+	MD5_body_for_thread(t, x, out)
+static MAYBE_INLINE_BODY void MD5_body_for_thread(int t,
+	MD5_word x[15], MD5_word out[4])
+#else
+static MAYBE_INLINE_BODY void MD5_body(MD5_word x[15], MD5_word out[4])
+#endif
 {
 	MD5_word a, b = Cb, c = Cc, d;
 
@@ -687,8 +573,16 @@ static void MD5_body(MD5_word x[15], MD5_word out[4])
 
 #else
 
-static void MD5_body(MD5_word x0[15], MD5_word x1[15],
+#if MD5_std_mt
+#define MD5_body(x0, x1, out0, out1) \
+	MD5_body_for_thread(t, x0, x1, out0, out1)
+static MAYBE_INLINE_BODY void MD5_body_for_thread(int t,
+	MD5_word x0[15], MD5_word x1[15],
 	MD5_word out0[4], MD5_word out1[4])
+#else
+static MAYBE_INLINE_BODY void MD5_body(MD5_word x0[15], MD5_word x1[15],
+	MD5_word out0[4], MD5_word out1[4])
+#endif
 {
 	MD5_word a0, b0 = Cb, c0 = Cc, d0;
 	MD5_word a1, b1, c1, d1;
@@ -862,6 +756,217 @@ static void MD5_body(MD5_word x0[15], MD5_word x1[15],
 
 #endif
 
+#if MD5_std_mt
+static MAYBE_INLINE void MD5_std_crypt_for_thread(int t)
+#else
+void MD5_std_crypt(int count)
+#endif
+{
+	int length, index, mask;
+	MD5_pattern *line;
+#if ARCH_LITTLE_ENDIAN
+	MD5_word *last0;
+#endif
+#if MD5_X2
+	MD5_pool *key;
+#if ARCH_LITTLE_ENDIAN
+	MD5_word *last1;
+#endif
+#endif
+
+#if MD5_X2
+	for (index = 0, key = pool; index < MD5_N; index++, key++) {
+#else
+#define index	0
+#define key	pool
+#endif
+		memcpy(key->o.ps.b, key->o.p.b, key->l.p);
+		memcpy(&key->o.ps.b[key->l.p], key->s, key->l.s);
+		key->l.ps = key->l.p + key->l.s;
+		memcpy(&key->o.ps.b[key->l.ps + 16], PADDING,
+			40 - key->l.ps);
+		key->o.ps.w[14] = (key->l.ps + 16) << 3;
+
+		memcpy(key->o.psp.b, key->o.ps.b, key->l.ps);
+		memcpy(&key->o.psp.b[key->l.ps], key->o.p.b, key->l.p);
+		key->l.psp = key->l.ps + key->l.p;
+		memcpy(&key->o.psp.b[key->l.psp + 16], PADDING,
+			40 - key->l.psp);
+		key->o.psp.w[14] = (key->l.psp + 16) << 3;
+
+		memcpy(&key->e.sp.b[16], key->s, key->l.s);
+		memcpy(&key->e.sp.b[16 + key->l.s], key->o.p.b,
+			key->l.p);
+		memcpy(&key->e.sp.b[16 + key->l.ps], PADDING,
+			40 - key->l.ps);
+		key->e.sp.w[14] = (key->l.ps + 16) << 3;
+		MD5_swap(key->e.sp.w, key->e.sp.w, 14);
+
+		memcpy(&key->e.spp.b[16], key->s, key->l.s);
+		memcpy(&key->e.spp.b[16 + key->l.s], key->o.pp.b,
+			key->l.pp);
+		memcpy(&key->e.spp.b[16 + key->l.psp], PADDING,
+			40 - key->l.psp);
+		key->e.spp.w[14] = (key->l.psp + 16) << 3;
+		MD5_swap(key->e.spp.w, key->e.spp.w, 14);
+
+		order[0][index].length = key->l.psp;
+		order[2][index].length = key->l.psp;
+		order[3][index].length = key->l.ps;
+		order[5][index].length = key->l.psp;
+		order[6][index].length = key->l.psp;
+		order[8][index].length = key->l.psp;
+		order[9][index].length = key->l.psp;
+		order[11][index].length = key->l.psp;
+		order[12][index].length = key->l.psp;
+		order[14][index].length = key->l.psp;
+		order[15][index].length = key->l.psp;
+		order[17][index].length = key->l.ps;
+		order[18][index].length = key->l.psp;
+		order[20][index].length = key->l.psp;
+
+		memcpy(&block[index], key->o.psp.b, key->l.psp);
+		memcpy(&block[index].b[key->l.psp], PADDING, 56 - key->l.psp);
+		block[index].w[14] = key->l.psp << 3;
+		MD5_swap(block[index].w, block[index].w, 14);
+#if MD5_X2
+	}
+
+	MD5_body(block[0].w, block[1].w, MD5_out[0], MD5_out[1]);
+	MD5_swap(MD5_out[0], MD5_out[0], 8);
+#else
+	MD5_body(block[0].w, MD5_out[0]);
+	MD5_swap(MD5_out[0], MD5_out[0], 4);
+#endif
+
+#if MD5_X2
+	for (index = 0, key = pool; index < MD5_N; index++, key++) {
+#endif
+		memcpy(&block[index], key->o.p.b, key->l.p);
+		memcpy(&block[index].b[key->l.p], prefix, prelen);
+		memcpy(&block[index].b[key->l.p + prelen], key->s, key->l.s);
+		memcpy(&block[index].b[key->l.ps + prelen],
+			MD5_out[index], key->l.p);
+		length = key->l.psp + prelen;
+		if ((mask = key->l.p))
+		do {
+			block[index].b[length++] =
+				(mask & 1) ? 0 : key->o.p.b[0];
+		} while (mask >>= 1);
+		memcpy(&block[index].b[length], PADDING, 56 - length);
+		block[index].w[14] = length << 3;
+		MD5_swap(block[index].w, block[index].w, 14);
+#if MD5_X2
+	}
+#else
+#undef index
+#undef key
+#endif
+
+#if MD5_X2
+	MD5_body(block[0].w, block[1].w,
+		order[0][0].even->w, order[0][1].even->w);
+#else
+	MD5_body(block[0].w, order[0][0].even->w);
+#endif
+
+	index = 500; line = order[0];
+	do {
+#if ARCH_LITTLE_ENDIAN
+#if ARCH_ALLOWS_UNALIGNED
+#if MD5_X2
+		MD5_body(line[0].even->w, line[1].even->w,
+			(MD5_word *)&line[0].odd->b[line[0].length],
+			(MD5_word *)&line[1].odd->b[line[1].length]);
+#else
+		MD5_body(line[0].even->w,
+			(MD5_word *)&line[0].odd->b[line[0].length]);
+#endif
+#else
+#if MD5_X2
+		MD5_body(line[0].even->w, line[1].even->w,
+			MD5_out[0], MD5_out[1]);
+		memcpy(&line[0].odd->b[line[0].length], MD5_out[0], 16);
+		memcpy(&line[1].odd->b[line[1].length], MD5_out[1], 16);
+#else
+		if (((ARCH_WORD)&line[0].odd->b[line[0].length]) & 3) {
+			MD5_body(line[0].even->w, MD5_out[0]);
+			memcpy(&line[0].odd->b[line[0].length],
+				MD5_out[0], 16);
+		} else {
+			MD5_body(line[0].even->w,
+				(MD5_word *)&line[0].odd->b[line[0].length]);
+		}
+#endif
+#endif
+		last0 = line[0].odd->w;
+#if MD5_X2
+		last1 = line[1].odd->w;
+		if ((line += 2) > &order[20][MD5_N - 1]) line = order[0];
+		MD5_body(last0, last1, line[0].even->w, line[1].even->w);
+#else
+		if (++line > &order[20][0]) line = order[0];
+		MD5_body(last0, line[0].even->w);
+#endif
+#else
+#if MD5_X2
+		MD5_body(line[0].even->w, line[1].even->w,
+			MD5_out[0], MD5_out[1]);
+		MD5_swap(MD5_out[0], MD5_out[0], 8);
+#else
+		MD5_body(line[0].even->w, MD5_out[0]);
+		MD5_swap(MD5_out[0], MD5_out[0], 4);
+#endif
+		memcpy(&line[0].odd->b[line[0].length], MD5_out[0], 16);
+#if MD5_X2
+		memcpy(&line[1].odd->b[line[1].length], MD5_out[1], 16);
+#endif
+		MD5_swap(block[0].w, line[0].odd->w, 14);
+		block[0].w[14] = line[0].odd->w[14];
+#if MD5_X2
+		MD5_swap(block[1].w, line[1].odd->w, 14);
+		block[1].w[14] = line[1].odd->w[14];
+		if ((line += 2) > &order[20][MD5_N - 1]) line = order[0];
+		MD5_body(block[0].w, block[1].w,
+			line[0].even->w, line[1].even->w);
+#else
+		if (++line > &order[20][0]) line = order[0];
+		MD5_body(block[0].w, line[0].even->w);
+#endif
+#endif
+	} while (--index);
+
+	memcpy(MD5_out[0], line[0].even, 16);
+#if MD5_X2
+	memcpy(MD5_out[1], line[1].even, 16);
+#endif
+}
+
+#if MD5_std_mt
+void MD5_std_crypt(int count)
+{
+#if MD5_std_mt
+	int t, n = (count + (MD5_N - 1)) / MD5_N;
+#endif
+
+#ifdef _OPENMP
+#pragma omp parallel for default(none) private(t) shared(n, salt_changed, saved_salt)
+#endif
+	for_each_t(n) {
+/*
+ * We could move the salt_changed check out of the parallel region (and have
+ * two specialized parallel regions instead), but MD5_std_crypt_for_thread()
+ * does so much work that the salt_changed check is negligible.
+ */
+		if (salt_changed)
+			MD5_std_set_salt_for_thread(t, saved_salt);
+		MD5_std_crypt_for_thread(t);
+	}
+
+	salt_changed = 0;
+}
+#endif
+
 char *MD5_std_get_salt(char *ciphertext)
 {
 	static char out[9];
@@ -916,7 +1021,10 @@ MD5_word *MD5_std_get_binary(char *ciphertext)
 		(MD5_word)atoi64[ARCH_INDEX(pos[0])] |
 		((MD5_word)atoi64[ARCH_INDEX(pos[1])] << 6);
 
+#undef OOFFOOFF
+#define OOFFOOFF 0x00ff00ff
 	MD5_swap(out.w, out.w, 4);
+#undef OOFFOOFF
 
 	return out.w;
 }
