@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-98,2003 by Solar Designer
+ * Copyright (c) 1996-98,2003,2004 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 #include <stdio.h>
@@ -37,8 +42,6 @@ static void do_incremental_pass(struct db_main *db)
 
 void do_batch_crack(struct db_main *db)
 {
-	status_init(NULL, 1);
-
 	switch (status.pass) {
 	case 0:
 	case 1:
