@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001,2003,2004,2006,2008-2010 by Solar Designer
+ * Copyright (c) 1996-2001,2003,2004,2006,2008-2012 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 #ifdef __ultrix__
@@ -8,6 +13,9 @@
 #define _POSIX_SOURCE
 #endif
 
+#define NEED_OS_TIMER
+#include "os.h"
+
 #ifdef _SCO_C_DIALECT
 #include <limits.h>
 #endif
@@ -95,7 +103,7 @@ char *benchmark_format(struct fmt_main *format, int salts,
 #endif
 	struct tms buf;
 	clock_t start_real, start_virtual, end_real, end_virtual;
-	unsigned ARCH_WORD count;
+	int64 crypts;
 	char *ciphertext;
 	void *salt, *two_salts[2];
 	int index, max;
@@ -117,9 +125,14 @@ char *benchmark_format(struct fmt_main *format, int salts,
 	for (index = 0; index < 2; index++) {
 		two_salts[index] = mem_alloc(format->params.salt_size);
 
-		if ((ciphertext = format->params.tests[index].ciphertext))
+		if ((ciphertext = format->params.tests[index].ciphertext)) {
+			char **fields = format->params.tests[index].fields;
+			if (!fields[1])
+				fields[1] = ciphertext;
+			ciphertext = format->methods.split(
+			    format->methods.prepare(fields, format), 0, format);
 			salt = format->methods.salt(ciphertext);
-		else
+		} else
 			salt = two_salts[0];
 
 		memcpy(two_salts[index], salt, format->params.salt_size);
@@ -145,12 +158,12 @@ char *benchmark_format(struct fmt_main *format, int salts,
 	if (benchmark_time > 3600)
 		benchmark_time = 3600;
 
-/* In the future, "zero time" may mean self-tests without benchmarks */
-	if (!benchmark_time)
-		benchmark_time = 1;
-
 #if OS_TIMER
-	it.it_value.tv_sec = benchmark_time;
+	if (!(it.it_value.tv_sec = benchmark_time)) {
+/* Use exactly one tick for reasonable precision, but no less than 1 ms */
+		if ((it.it_value.tv_usec = 1000000 / clk_tck) < 1000)
+			it.it_value.tv_usec = 1000; /* 1 ms */
+	}
 	if (setitimer(ITIMER_REAL, &it, NULL)) pexit("setitimer");
 #else
 	sig_timer_emu_init(benchmark_time * clk_tck);
@@ -159,11 +172,13 @@ char *benchmark_format(struct fmt_main *format, int salts,
 	start_real = times(&buf);
 	start_virtual = buf.tms_utime + buf.tms_stime;
 	start_virtual += buf.tms_cutime + buf.tms_cstime;
-	count = 0;
+	crypts.lo = crypts.hi = 0;
 
 	index = salts;
 	max = format->params.max_keys_per_crypt;
 	do {
+		int count = max;
+
 		if (!--index) {
 			index = salts;
 			if (!(++current)->ciphertext)
@@ -172,23 +187,25 @@ char *benchmark_format(struct fmt_main *format, int salts,
 		}
 
 		if (salts > 1) format->methods.set_salt(two_salts[index & 1]);
-		format->methods.crypt_all(max);
-		format->methods.cmp_all(binary, max);
+		format->methods.cmp_all(binary,
+		    format->methods.crypt_all(&count, NULL));
 
-		count++;
+		add32to64(&crypts, count);
 #if !OS_TIMER
 		sig_timer_emu_tick();
 #endif
 	} while (bench_running && !event_abort);
 
 	end_real = times(&buf);
+	if (end_real == start_real) end_real++;
+
 	end_virtual = buf.tms_utime + buf.tms_stime;
 	end_virtual += buf.tms_cutime + buf.tms_cstime;
 	if (end_virtual == start_virtual) end_virtual++;
 
 	results->real = end_real - start_real;
 	results->virtual = end_virtual - start_virtual;
-	results->count = count * max;
+	results->crypts = crypts;
 
 	for (index = 0; index < 2; index++)
 		MEM_FREE(two_salts[index]);
@@ -196,12 +213,12 @@ char *benchmark_format(struct fmt_main *format, int salts,
 	return event_abort ? "" : NULL;
 }
 
-void benchmark_cps(unsigned ARCH_WORD count, clock_t time, char *buffer)
+void benchmark_cps(int64 *crypts, clock_t time, char *buffer)
 {
 	unsigned int cps_hi, cps_lo;
 	int64 tmp;
 
-	tmp.lo = count; tmp.hi = 0;
+	tmp = *crypts;
 	mul64by32(&tmp, clk_tck);
 	cps_hi = div64by32lo(&tmp, time);
 
@@ -225,13 +242,19 @@ int benchmark_all(void)
 	char s_real[64], s_virtual[64];
 	unsigned int total, failed;
 
+	if (!benchmark_time)
+		puts("Warning: doing quick benchmarking - "
+		    "the performance numbers will be inaccurate");
+
 	total = failed = 0;
 	if ((format = fmt_list))
 	do {
-		printf("Benchmarking: %s%s [%s]... ",
-			format->params.format_name,
-			format->params.benchmark_comment,
-			format->params.algorithm_name);
+		printf("Benchmarking: %s%s%s%s [%s]... ",
+		    format->params.label,
+		    format->params.format_name[0] ? ", " : "",
+		    format->params.format_name,
+		    format->params.benchmark_comment,
+		    format->params.algorithm_name);
 		fflush(stdout);
 
 		switch (format->params.benchmark_length) {
@@ -257,21 +280,21 @@ int benchmark_all(void)
 		    &results_m))) {
 			puts(result);
 			failed++;
-			continue;
+			goto next;
 		}
 
 		if (msg_1)
 		if ((result = benchmark_format(format, 1, &results_1))) {
 			puts(result);
 			failed++;
-			continue;
+			goto next;
 		}
 
 		puts("DONE");
 
-		benchmark_cps(results_m.count, results_m.real, s_real);
-		benchmark_cps(results_m.count, results_m.virtual, s_virtual);
-#if !defined(__DJGPP__) && !defined(__CYGWIN32__) && !defined(__BEOS__)
+		benchmark_cps(&results_m.crypts, results_m.real, s_real);
+		benchmark_cps(&results_m.crypts, results_m.virtual, s_virtual);
+#if !defined(__DJGPP__) && !defined(__BEOS__)
 		printf("%s:\t%s c/s real, %s c/s virtual\n",
 			msg_m, s_real, s_virtual);
 #else
@@ -281,18 +304,21 @@ int benchmark_all(void)
 
 		if (!msg_1) {
 			putchar('\n');
-			continue;
+			goto next;
 		}
 
-		benchmark_cps(results_1.count, results_1.real, s_real);
-		benchmark_cps(results_1.count, results_1.virtual, s_virtual);
-#if !defined(__DJGPP__) && !defined(__CYGWIN32__) && !defined(__BEOS__)
+		benchmark_cps(&results_1.crypts, results_1.real, s_real);
+		benchmark_cps(&results_1.crypts, results_1.virtual, s_virtual);
+#if !defined(__DJGPP__) && !defined(__BEOS__)
 		printf("%s:\t%s c/s real, %s c/s virtual\n\n",
 			msg_1, s_real, s_virtual);
 #else
 		printf("%s:\t%s c/s\n\n",
 			msg_1, s_real);
 #endif
+
+next:
+		fmt_done(format);
 	} while ((format = format->next) && !event_abort);
 
 	if (failed && total > 1 && !event_abort)
