@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-99,2003,2006 by Solar Designer
+ * Copyright (c) 1996-99,2003,2006,2011-2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -12,6 +17,9 @@
 #define _POSIX_SOURCE
 #endif
 
+#define NEED_OS_FORK
+#include "os.h"
+
 #ifdef _SCO_C_DIALECT
 #include <limits.h>
 #endif
@@ -26,6 +34,12 @@
 
 extern struct fmt_main fmt_DES, fmt_MD5, fmt_BF;
 
+int john_main_process = 0;
+#if OS_FORK
+int john_child_count = 0;
+int *john_child_pids = NULL;
+#endif
+
 int main(int argc, char **argv)
 {
 	struct fmt_main *format;
@@ -53,10 +67,12 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
-	fprintf(stderr, "Benchmarking: %s%s [%s]... ",
-		format->params.format_name,
-		format->params.benchmark_comment,
-		format->params.algorithm_name);
+	fprintf(stderr, "Benchmarking: %s%s%s%s [%s]... ",
+	    format->params.label,
+	    format->params.format_name[0] ? ", " : "",
+	    format->params.format_name,
+	    format->params.benchmark_comment,
+	    format->params.algorithm_name);
 
 	common_init();
 
@@ -65,17 +81,23 @@ int main(int argc, char **argv)
 
 		fprintf(stderr, "FAILED\n");
 	} else {
-		tmp.lo = results.count; tmp.hi = 0;
+		tmp = results.crypts;
 		mul64by32(&tmp, clk_tck * 10);
+#ifdef _OPENMP
+		virtual = div64by32lo(&tmp, results.real);
+#else
 		virtual = div64by32lo(&tmp, results.virtual);
+#endif
 
-		benchmark_cps(results.count, results.real, s_real);
-		benchmark_cps(results.count, results.virtual, s_virtual);
+		benchmark_cps(&results.crypts, results.real, s_real);
+		benchmark_cps(&results.crypts, results.virtual, s_virtual);
 
 		fprintf(stderr, "%s c/s real, %s c/s virtual\n",
 			s_real, s_virtual);
 	}
 
+	fmt_done(format);
+
 	printf("%lu\n", virtual);
 
 	return virtual ? 0 : 1;
