@@ -1,9 +1,14 @@
 /*
- * This file is part of John the Ripper password cracker,
- * Copyright (c) 2009-2011 by Solar Designer
- *
  * Generic crypt(3) support, as well as support for glibc's crypt_r(3) and
  * Solaris' MT-safe crypt(3C) with OpenMP parallelization.
+ *
+ * This file is part of John the Ripper password cracker,
+ * Copyright (c) 2009-2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 #define _XOPEN_SOURCE 4 /* for crypt(3) */
@@ -11,6 +16,7 @@
 #define _XOPEN_VERSION 4
 #define _XPG4_2
 #define _GNU_SOURCE /* for crypt_r(3) */
+#include <stdio.h>
 #include <string.h>
 #if defined(_OPENMP) && defined(__GLIBC__)
 #include <crypt.h>
@@ -25,6 +31,7 @@
 #include "memory.h"
 #include "common.h"
 #include "formats.h"
+#include "loader.h"
 
 #define FORMAT_LABEL			"crypt"
 #define FORMAT_NAME			"generic crypt(3)"
@@ -36,7 +43,9 @@
 #define PLAINTEXT_LENGTH		72
 
 #define BINARY_SIZE			128
+#define BINARY_ALIGN			1
 #define SALT_SIZE			BINARY_SIZE
+#define SALT_ALIGN			1
 
 #define MIN_KEYS_PER_CRYPT		96
 #define MAX_KEYS_PER_CRYPT		96
@@ -61,7 +70,7 @@ static char crypt_out[MAX_KEYS_PER_CRYPT][BINARY_SIZE];
 static struct crypt_data *crypt_data[MAX_THREADS];
 #endif
 
-static int valid(char *ciphertext)
+static int valid(char *ciphertext, struct fmt_main *self)
 {
 	int length, count_base64, id, pw_length;
 	char pw[PLAINTEXT_LENGTH + 1], *new_ciphertext;
@@ -137,15 +146,15 @@ static int valid(char *ciphertext)
 	new_ciphertext = crypt(pw, ciphertext);
 #endif
 
-	if (strlen(new_ciphertext) == length &&
+	if (new_ciphertext && strlen(new_ciphertext) == length &&
 	    !strncmp(new_ciphertext, ciphertext, 2)) {
 		sup_length[length] = 1;
 		sup_id[id] = 1;
 		return 1;
 	}
 
-	if (id != 10)
-		fprintf(stderr, "Generic crypt(3) module: "
+	if (id != 10 && !ldr_in_pot)
+		fprintf(stderr, "Warning: "
 		    "hash encoding string length %d, type id %c%c\n"
 		    "appears to be unsupported on this system; "
 		    "will not load such hashes.\n",
@@ -201,7 +210,10 @@ static void *salt(char *ciphertext)
 		if (length == 59 && !strncmp(ciphertext, "$2$", 3))
 			cut = 28;
 		else
-		if (length == 60 && !strncmp(ciphertext, "$2a$", 4))
+		if (length == 60 &&
+		    (!strncmp(ciphertext, "$2a$", 4) ||
+		    !strncmp(ciphertext, "$2x$", 4) ||
+		    !strncmp(ciphertext, "$2y$", 4)))
 			cut = 29;
 		else
 		if (length >= 27 &&
@@ -317,7 +329,7 @@ static int salt_hash(void *salt)
 	h ^= (unsigned char)atoi64[ARCH_INDEX(((char *)salt)[i - 1])];
 	h ^= ((unsigned char *)salt)[i];
 
-	return h & 0x3FF;
+	return h & (SALT_HASH_SIZE - 1);
 }
 
 static void set_salt(void *salt)
@@ -335,12 +347,14 @@ static char *get_key(int index)
 	return saved_key[index];
 }
 
-static void crypt_all(int count)
+static int crypt_all(int *pcount, struct db_salt *salt)
 {
+	static int warned = 0;
+	int count = *pcount;
 	int index;
 
 #if defined(_OPENMP) && defined(__GLIBC__)
-#pragma omp parallel for default(none) private(index) shared(count, crypt_out, saved_key, saved_salt, crypt_data)
+#pragma omp parallel for default(none) private(index) shared(warned, count, crypt_out, saved_key, saved_salt, crypt_data, stderr)
 	for (index = 0; index < count; index++) {
 		char *hash;
 		int t = omp_get_thread_num();
@@ -368,6 +382,15 @@ static void crypt_all(int count)
 			memset(&data, 0, sizeof(data));
 			hash = crypt_r(saved_key[index], saved_salt, &data);
 		}
+		if (!hash) {
+#pragma omp critical
+			if (!warned) {
+				fprintf(stderr,
+				    "Warning: crypt_r() returned NULL\n");
+				warned = 1;
+			}
+			hash = "";
+		}
 		strnzcpy(crypt_out[index], hash, BINARY_SIZE);
 	}
 #else
@@ -379,19 +402,39 @@ static void crypt_all(int count)
  * modern hash types, the function is actually able to compute multiple hashes
  * in parallel by different threads (and the performance for some hash types is
  * reasonable).  Overall, this code is reasonable to use for SHA-crypt and
- * SunMD5 hashes, which are not yet supported by John natively.
+ * SunMD5 hashes, which are not yet supported by non-jumbo John natively.
  */
-#pragma omp parallel for default(none) private(index) shared(count, crypt_out, saved_key, saved_salt)
+#pragma omp parallel for default(none) private(index) shared(warned, count, crypt_out, saved_key, saved_salt, stderr)
 #endif
-	for (index = 0; index < count; index++)
-		strnzcpy(crypt_out[index], crypt(saved_key[index], saved_salt),
-		    BINARY_SIZE);
+	for (index = 0; index < count; index++) {
+		char *hash = crypt(saved_key[index], saved_salt);
+		if (!hash) {
+#if defined(_OPENMP) && defined(__sun)
+#pragma omp critical
 #endif
+			if (!warned) {
+				fprintf(stderr,
+				    "Warning: crypt() returned NULL\n");
+				warned = 1;
+			}
+			hash = "";
+		}
+		strnzcpy(crypt_out[index], hash, BINARY_SIZE);
+	}
+#endif
+
+	return count;
 }
 
 static int cmp_all(void *binary, int count)
 {
-	return 1;
+	int index;
+
+	for (index = 0; index < count; index++)
+		if (!strcmp((char *)binary, crypt_out[index]))
+			return 1;
+
+	return 0;
 }
 
 static int cmp_one(void *binary, int index)
@@ -413,23 +456,31 @@ struct fmt_main fmt_crypt = {
 		BENCHMARK_LENGTH,
 		PLAINTEXT_LENGTH,
 		BINARY_SIZE,
+		BINARY_ALIGN,
 		SALT_SIZE,
+		SALT_ALIGN,
 		MIN_KEYS_PER_CRYPT,
 		MAX_KEYS_PER_CRYPT,
 		FMT_CASE | FMT_8_BIT | FMT_OMP,
 		tests
 	}, {
 		fmt_default_init,
+		fmt_default_done,
+		fmt_default_reset,
+		fmt_default_prepare,
 		valid,
 		fmt_default_split,
 		binary,
 		salt,
+		fmt_default_source,
 		{
 			binary_hash_0,
 			binary_hash_1,
 			binary_hash_2,
 			binary_hash_3,
-			binary_hash_4
+			binary_hash_4,
+			NULL,
+			NULL
 		},
 		salt_hash,
 		set_salt,
@@ -442,7 +493,9 @@ struct fmt_main fmt_crypt = {
 			get_hash_1,
 			get_hash_2,
 			get_hash_3,
-			get_hash_4
+			get_hash_4,
+			NULL,
+			NULL
 		},
 		cmp_all,
 		cmp_one,
