@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-99,2005,2009 by Solar Designer
+ * Copyright (c) 1996-99,2005,2009,2011,2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -19,6 +24,18 @@ typedef unsigned short ARCH_WORD_32;
 typedef unsigned int ARCH_WORD_32;
 #endif
 
+#ifdef __GNUC__
+#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1)
+#define MAYBE_INLINE __attribute__((always_inline)) __inline__
+#else
+#define MAYBE_INLINE __inline__
+#endif
+#elif __STDC_VERSION__ >= 199901L
+#define MAYBE_INLINE inline
+#else
+#define MAYBE_INLINE
+#endif
+
 #if ((__GNUC__ == 2) && (__GNUC_MINOR__ >= 7)) || (__GNUC__ > 2)
 #define CC_CACHE_ALIGN \
 	__attribute__ ((aligned (MEM_ALIGN_CACHE)))
