@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2000,2003,2005,2011 by Solar Designer
+ * Copyright (c) 1996-2000,2003,2005,2011-2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 #include <stdio.h>
@@ -8,6 +13,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
+#include <assert.h>
 
 #include "arch.h"
 #include "params.h"
@@ -221,22 +227,24 @@ static char *c_gettoken(void)
 	return token;
 }
 
-static int c_getint(char *token)
+static c_int c_getint(char *token)
 {
-	int value;
+	c_int value;
 	long l_value;
 	char *error;
 
 	if (token[0] == '\'') {
-		if ((value = c_getchar(1)) == '\'')
+		if ((value = (unsigned char)c_getchar(1)) == '\'')
 			c_errno = C_ERROR_UNEXPECTED;
 		else
-			if (value == '\\') value = c_getchar(1);
-		if (c_getchar(1) != '\'') c_errno = C_ERROR_UNEXPECTED;
+			if (value == '\\')
+				value = (unsigned char)c_getchar(1);
+		if (c_getchar(1) != '\'')
+			c_errno = C_ERROR_UNEXPECTED;
 	} else {
 		errno = 0;
 		l_value = strtol(token, &error, 0);
-		value = (int)l_value;
+		value = (c_int)l_value;
 		if (errno == ERANGE || (long)value != l_value)
 			c_errno = C_ERROR_RANGE;
 		else
@@ -422,7 +430,7 @@ static int c_define(char term, struct c_ident **vars, struct c_ident *globals)
 {
 	char *token;
 	char c;
-	int size;
+	c_int size;
 
 	c_expect(' ');
 	token = c_gettoken();
@@ -809,15 +817,11 @@ static int c_block(char term, struct c_ident *vars)
 	return c_errno;
 }
 
-#if defined(__GNUC__) && !defined(PRINT_INSNS)
-static void c_direct(union c_insn *addr);
-#endif
-
 int c_compile(int (*ext_getchar)(void), void (*ext_rewind)(void),
 	struct c_ident *externs)
 {
 #if defined(__GNUC__) && !defined(PRINT_INSNS)
-	c_direct(NULL);
+	c_execute_fast(NULL);
 #endif
 
 	c_ext_getchar = ext_getchar;
@@ -843,43 +847,54 @@ int c_compile(int (*ext_getchar)(void), void (*ext_rewind)(void),
 
 		c_code_start = mem_alloc((size_t)c_code_ptr);
 		c_data_start = mem_alloc((size_t)c_data_ptr);
+		memset(c_data_start, 0, (size_t)c_data_ptr);
 	}
 
 	return c_errno;
 }
 
-struct c_ident *c_lookup(char *name)
+void *c_lookup(char *name)
 {
-	return c_find_ident(c_funcs, NULL, name);
+	struct c_ident *f = c_find_ident(c_funcs, NULL, name);
+	if (f)
+		return f->addr;
+	return NULL;
 }
 
-void c_execute(struct c_ident *fn)
-{
-	if (!fn) return;
+#if !defined(__GNUC__) || defined(PRINT_INSNS)
 
-#if defined(__GNUC__) && !defined(PRINT_INSNS)
-	c_direct(fn->addr);
-#else
+void c_execute_fast(void *addr)
+{
 	c_stack[0].pc = NULL;
 	c_sp = &c_stack[2];
 
-	c_pc = fn->addr;
+	c_pc = addr;
 	do {
-		void (*op)(void) = (c_pc++)->op;
 #ifdef PRINT_INSNS
+		void (*op)(void) = (c_pc++)->op;
 		int i = 0;
 		while (c_ops[i].op != op && c_ops[i].prec >= 0)
 			i++;
 		fprintf(stderr, "op: %s\n", c_ops[i].name);
-#endif
 		op();
-	} while (c_pc);
+#else
+		(c_pc++)->op();
+		if (!c_pc)
+			break;
+		(c_pc++)->op();
+		if (!c_pc)
+			break;
+		(c_pc++)->op();
+		if (!c_pc)
+			break;
+		(c_pc++)->op();
 #endif
+	} while (c_pc);
 }
 
-#if defined(__GNUC__) && !defined(PRINT_INSNS)
+#else
 
-static void c_direct(union c_insn *addr)
+void c_execute_fast(void *addr)
 {
 	union c_insn *pc = addr;
 	union c_insn *sp = c_stack;
@@ -925,9 +940,15 @@ static void c_direct(union c_insn *addr)
 		&&op_dec_r
 	};
 
+#if __GNUC__ >= 3
+	if (__builtin_expect(addr == NULL, 0)) {
+#else
 	if (!addr) {
+#endif
 		int op = 0;
 
+		assert(c_op_return != &&op_return); /* Don't do this twice */
+
 		c_op_return = &&op_return;
 		c_op_bz = &&op_bz;
 		c_op_ba = &&op_ba;
@@ -959,12 +980,15 @@ op_return:
 	return;
 
 op_bz:
+	sp -= 2;
+#if __GNUC__ >= 3
+	if (__builtin_expect(imm != 0, 1)) {
+#else
 	if (imm) {
+#endif
 		pc += 2;
-		sp -= 2;
 		goto *(pc - 1)->op;
 	}
-	sp -= 2;
 
 op_ba:
 	pc = pc->pc;
