@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001 by Solar Designer
+ * Copyright (c) 1996-2001,2011 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -10,13 +15,22 @@
 #ifndef _JOHN_EXTERNAL_H
 #define _JOHN_EXTERNAL_H
 
+#include "compiler.h"
 #include "loader.h"
 
+#define EXT_REQ_GENERATE		0x00000001
+#define EXT_REQ_FILTER			0x00000002
+#define EXT_USES_GENERATE		0x00010000
+#define EXT_USES_FILTER			0x00020000
+
+extern unsigned int ext_flags;
+
+extern c_int ext_abort, ext_status;
+
 /*
  * Defined for use in the ext_filter() macro, below.
  */
-extern char *ext_mode;
-extern struct c_ident *f_filter;
+extern void *f_filter;
 
 /*
  * Initializes an external mode.
@@ -27,7 +41,7 @@ extern void ext_init(char *mode);
  * Calls an external word filter. Returns 0 if the word is rejected.
  */
 #define ext_filter(word) \
-	(!ext_mode || !f_filter || ext_filter_body(word, word))
+	(!f_filter || ext_filter_body(word, word))
 
 /*
  * The actual implementation of ext_filter(); use the macro instead.
