@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2000,2003,2005,2010,2011 by Solar Designer
+ * Copyright (c) 1996-2000,2003,2005,2010-2012 by Solar Designer
  */
 
 #include <stdio.h>
@@ -21,6 +21,7 @@
 
 #ifdef HAVE_CRYPT
 extern struct fmt_main fmt_crypt;
+int ldr_in_pot = 0;
 #endif
 
 /*
@@ -90,8 +91,8 @@ void ldr_init_database(struct db_main *db, struct db_options *options)
 {
 	db->loaded = 0;
 
-	db->options = mem_alloc_copy(sizeof(struct db_options),
-		MEM_ALIGN_WORD, options);
+	db->options = mem_alloc_copy(options,
+	    sizeof(struct db_options), MEM_ALIGN_WORD);
 
 	db->salts = NULL;
 
@@ -137,9 +138,9 @@ void ldr_init_database(struct db_main *db, struct db_options *options)
 static void ldr_init_password_hash(struct db_main *db)
 {
 	int (*func)(void *binary);
-	int size = PASSWORD_HASH_SIZES - 1;
+	int size = PASSWORD_HASH_SIZE_FOR_LDR;
 
-	if (mem_saving_level >= 2)
+	if (size > 0 && mem_saving_level >= 2)
 		size--;
 
 	do {
@@ -216,15 +217,15 @@ static int ldr_split_line(char **login, char **ciphertext,
 	struct db_options *options, char *line)
 {
 	struct fmt_main *alt;
-	char *uid = NULL, *gid = NULL, *shell = NULL;
-	int retval;
+	char *fields[10], *uid, *gid, *shell;
+	int i, retval;
 
-	*login = ldr_get_field(&line);
-	*ciphertext = ldr_get_field(&line);
+	fields[0] = *login = ldr_get_field(&line);
+	fields[1] = *ciphertext = ldr_get_field(&line);
 
 /* Check for NIS stuff */
 	if ((!strcmp(*login, "+") || !strncmp(*login, "+@", 2)) &&
-	    strlen(*ciphertext) < 13 && strncmp(*ciphertext, "$dummy$", 7))
+	    strlen(*ciphertext) < 10 && strncmp(*ciphertext, "$dummy$", 7))
 		return 0;
 
 	if (!**ciphertext && !line) {
@@ -236,9 +237,10 @@ static int ldr_split_line(char **login, char **ciphertext,
 		p += strlen(p) - 1;
 		while (p > *ciphertext && (*p == ' ' || *p == '\t')) p--;
 		p++;
-/* Some valid dummy hashes may be shorter than 13 characters, so don't subject
+/* Some valid dummy hashes may be shorter than 10 characters, so don't subject
  * them to the length checks. */
-		if (strncmp(*ciphertext, "$dummy$", 7)) {
+		if (strncmp(*ciphertext, "$dummy$", 7) &&
+		    p - *ciphertext != 10 /* not tripcode */) {
 /* Check for a special case: possibly a traditional crypt(3) hash with
  * whitespace in its invalid salt.  Only support such hashes at the very start
  * of a line (no leading whitespace other than the invalid salt). */
@@ -250,35 +252,62 @@ static int ldr_split_line(char **login, char **ciphertext,
 				return 0;
 		}
 		*p = 0;
-		*login = no_username;
+		fields[0] = *login = no_username;
+		fields[1] = *ciphertext;
 	}
 
-	if (source) strcpy(source, line ? line : "");
+	if (source)
+		strcpy(source, line ? line : "");
 
-	uid = ldr_get_field(&line);
+/*
+ * This check is just a loader performance optimization, so that we can parse
+ * fewer fields when we know we won't need the rest.  It should be revised or
+ * removed when there are formats that use higher-numbered fields in prepare().
+ */
+	if ((options->flags & DB_WORDS) || options->shells->head) {
+		/* Parse all fields */
+		for (i = 2; i < 10; i++)
+			fields[i] = ldr_get_field(&line);
+	} else {
+		/* Parse some fields only */
+		for (i = 2; i < 4; i++)
+			fields[i] = ldr_get_field(&line);
+		for (; i < 10; i++)
+			fields[i] = "/";
+	}
 
-	if (strlen(uid) == 32) {
-		char *tmp = *ciphertext;
-		*ciphertext = uid;
-		uid = tmp;
+	/* /etc/passwd */
+	uid = fields[2];
+	gid = fields[3];
+	*gecos = fields[4];
+	*home = fields[5];
+	shell = fields[6];
 
+	if (fields[5][0] != '/' &&
+	    ((!strcmp(fields[5], "0") && !strcmp(fields[6], "0")) ||
+	    fields[8][0] == '/' ||
+	    fields[9][0] == '/')) {
+		/* /etc/master.passwd */
+		*gecos = fields[7];
+		*home = fields[8];
+		shell = fields[9];
+	} else if (fields[3] - fields[2] == 32 + 1) {
+		/* PWDUMP */
+		uid = fields[1];
+		*ciphertext = fields[2];
 		if (!strncmp(*ciphertext, "NO PASSWORD", 11))
 			*ciphertext = "";
+		gid = shell = "";
+		*gecos = fields[4];
+		*home = fields[5];
 
-		if (source) sprintf(source, "%s:%s", uid, line);
-	}
-
-	if ((options->flags & DB_WORDS) || options->shells->head) {
-		gid = ldr_get_field(&line);
-		do {
-			*gecos = ldr_get_field(&line);
-			*home = ldr_get_field(&line);
-			shell = ldr_get_field(&line);
-		} while (!**gecos &&
-			!strcmp(*home, "0") && !strcmp(shell, "0"));
-	} else
-	if (options->groups->head) {
-		gid = ldr_get_field(&line);
+		/* Re-introduce the previously removed uid field */
+		if (source) {
+			int shift = strlen(uid);
+			memmove(source + shift + 1, source, strlen(source) + 1);
+			memcpy(source, uid, shift);
+			source[shift] = ':';
+		}
 	}
 
 	if (ldr_check_list(options->users, *login, uid)) return 0;
@@ -286,48 +315,60 @@ static int ldr_split_line(char **login, char **ciphertext,
 	if (ldr_check_shells(options->shells, shell)) return 0;
 
 	if (*format) {
-		int valid = (*format)->methods.valid(*ciphertext);
-		if (!valid) {
-			alt = fmt_list;
-			do {
-				if (alt == *format)
-					continue;
-				if (alt->params.flags & FMT_WARNED)
-					continue;
-#ifdef HAVE_CRYPT
-				if (alt == &fmt_crypt &&
-#ifdef __sun
-				    strncmp(*ciphertext, "$md5$", 5) &&
-				    strncmp(*ciphertext, "$md5,", 5) &&
-#endif
-				    strncmp(*ciphertext, "$5$", 3) &&
-				    strncmp(*ciphertext, "$6$", 3))
-					continue;
-#endif
-				if (alt->methods.valid(*ciphertext)) {
-					alt->params.flags |= FMT_WARNED;
-					fprintf(stderr,
-					    "Warning: only loading hashes "
-					    "of type \"%s\", but also saw "
-					    "type \"%s\"\n"
-					    "Use the "
-					    "\"--format=%s\" option to force "
-					    "loading hashes of that type "
-					    "instead\n",
-					    (*format)->params.label,
-					    alt->params.label,
-					    alt->params.label);
-					break;
-				}
-			} while ((alt = alt->next));
+		char *prepared;
+		int valid;
+
+		prepared = (*format)->methods.prepare(fields, *format);
+		if (prepared)
+			valid = (*format)->methods.valid(prepared, *format);
+		else
+			valid = 0;
+
+		if (valid) {
+			*ciphertext = prepared;
+			return valid;
 		}
-		return valid;
+
+		alt = fmt_list;
+		do {
+			if (alt == *format)
+				continue;
+			if (alt->params.flags & FMT_WARNED)
+				continue;
+#ifdef HAVE_CRYPT
+			if (alt == &fmt_crypt &&
+#ifdef __sun
+			    strncmp(*ciphertext, "$md5$", 5) &&
+			    strncmp(*ciphertext, "$md5,", 5) &&
+#endif
+			    strncmp(*ciphertext, "$5$", 3) &&
+			    strncmp(*ciphertext, "$6$", 3))
+				continue;
+#endif
+			prepared = alt->methods.prepare(fields, alt);
+			if (alt->methods.valid(prepared, alt)) {
+				alt->params.flags |= FMT_WARNED;
+				fprintf(stderr,
+				    "Warning: only loading hashes of type "
+				    "\"%s\", but also saw type \"%s\"\n"
+				    "Use the \"--format=%s\" option to force "
+				    "loading hashes of that type instead\n",
+				    (*format)->params.label,
+				    alt->params.label,
+				    alt->params.label);
+				break;
+			}
+		} while ((alt = alt->next));
+
+		return 0;
 	}
 
 	retval = -1;
 	if ((alt = fmt_list))
 	do {
+		char *prepared;
 		int valid;
+
 #ifdef HAVE_CRYPT
 /*
  * Only probe for support by the current system's crypt(3) if this is forced
@@ -345,11 +386,18 @@ static int ldr_split_line(char **login, char **ciphertext,
 		    strncmp(*ciphertext, "$6$", 3))
 			continue;
 #endif
-		if (!(valid = alt->methods.valid(*ciphertext)))
+
+		prepared = alt->methods.prepare(fields, alt);
+		if (!prepared)
 			continue;
+		valid = alt->methods.valid(prepared, alt);
+		if (!valid)
+			continue;
+
 		if (retval < 0) {
-			fmt_init(*format = alt);
 			retval = valid;
+			*ciphertext = prepared;
+			fmt_init(*format = alt);
 #ifdef LDR_WARN_AMBIGUOUS
 			if (!source) /* not --show */
 				continue;
@@ -450,7 +498,7 @@ static void ldr_load_pw_line(struct db_main *db, char *line)
 		ldr_init_password_hash(db);
 
 	for (index = 0; index < count; index++) {
-		piece = format->methods.split(ciphertext, index);
+		piece = format->methods.split(ciphertext, index, format);
 
 		binary = format->methods.binary(piece);
 		pw_hash = db->password_hash_func(binary);
@@ -459,9 +507,10 @@ static void ldr_load_pw_line(struct db_main *db, char *line)
 			int collisions = 0;
 			if ((current_pw = db->password_hash[pw_hash]))
 			do {
-				if (!memcmp(current_pw->binary, binary,
+				if (!memcmp(binary, current_pw->binary,
 				    format->params.binary_size) &&
-				    !strcmp(current_pw->source, piece)) {
+				    !strcmp(piece, format->methods.source(
+				    current_pw->source, current_pw->binary))) {
 					db->options->flags |= DB_NODUP;
 					break;
 				}
@@ -504,11 +553,12 @@ static void ldr_load_pw_line(struct db_main *db, char *line)
 				mem_alloc_tiny(salt_size, MEM_ALIGN_WORD);
 			current_salt->next = last_salt;
 
-			current_salt->salt = mem_alloc_copy(
-				format->params.salt_size, MEM_ALIGN_WORD,
-				salt);
+			current_salt->salt = mem_alloc_copy(salt,
+				format->params.salt_size,
+				format->params.salt_align);
 
 			current_salt->index = fmt_dummy_hash;
+			current_salt->bitmap = NULL;
 			current_salt->list = NULL;
 			current_salt->hash = &current_salt->list;
 			current_salt->hash_size = -1;
@@ -533,10 +583,19 @@ static void ldr_load_pw_line(struct db_main *db, char *line)
 		db->password_hash[pw_hash] = current_pw;
 		current_pw->next_hash = last_pw;
 
-		current_pw->binary = mem_alloc_copy(
-			format->params.binary_size, MEM_ALIGN_WORD, binary);
+/* If we're not going to use the source field for its usual purpose, see if we
+ * can pack the binary value in it. */
+		if (format->methods.source != fmt_default_source &&
+		    sizeof(current_pw->source) >= format->params.binary_size)
+			current_pw->binary = memcpy(&current_pw->source,
+				binary, format->params.binary_size);
+		else
+			current_pw->binary = mem_alloc_copy(binary,
+				format->params.binary_size,
+				format->params.binary_align);
 
-		current_pw->source = str_alloc_copy(piece);
+		if (format->methods.source == fmt_default_source)
+			current_pw->source = str_alloc_copy(piece);
 
 		if (db->options->flags & DB_WORDS) {
 			if (!words)
@@ -576,25 +635,36 @@ static void ldr_load_pot_line(struct db_main *db, char *line)
 	struct db_password *current;
 
 	ciphertext = ldr_get_field(&line);
-	if (format->methods.valid(ciphertext) != 1) return;
+	if (format->methods.valid(ciphertext, format) != 1) return;
 
-	ciphertext = format->methods.split(ciphertext, 0);
+	ciphertext = format->methods.split(ciphertext, 0, format);
 	binary = format->methods.binary(ciphertext);
 	hash = db->password_hash_func(binary);
 
 	if ((current = db->password_hash[hash]))
 	do {
-		if (current->binary && !memcmp(current->binary, binary,
-		    format->params.binary_size) &&
-		    !strcmp(current->source, ciphertext))
-			current->binary = NULL;
+		if (!current->binary) /* already marked for removal */
+			continue;
+		if (memcmp(binary, current->binary, format->params.binary_size))
+			continue;
+		if (strcmp(ciphertext,
+		    format->methods.source(current->source, current->binary)))
+			continue;
+		current->binary = NULL; /* mark for removal */
 	} while ((current = current->next_hash));
 }
 
 void ldr_load_pot_file(struct db_main *db, char *name)
 {
-	if (db->format)
+	if (db->format) {
+#ifdef HAVE_CRYPT
+		ldr_in_pot = 1;
+#endif
 		read_file(db, name, RF_ALLOW_MISSING, ldr_load_pot_line);
+#ifdef HAVE_CRYPT
+		ldr_in_pot = 0;
+#endif
+	}
 }
 
 /*
@@ -697,6 +767,7 @@ static void ldr_init_hash_for_salt(struct db_main *db, struct db_salt *salt)
 {
 	struct db_password *current;
 	int (*hash_func)(void *binary);
+	int bitmap_size, hash_size;
 	int hash;
 
 	if (salt->hash_size < 0) {
@@ -710,10 +781,21 @@ static void ldr_init_hash_for_salt(struct db_main *db, struct db_salt *salt)
 		return;
 	}
 
-	salt->hash = mem_alloc_tiny(password_hash_sizes[salt->hash_size] *
-	    sizeof(struct db_password *), MEM_ALIGN_WORD);
-	memset(salt->hash, 0, password_hash_sizes[salt->hash_size] *
-	    sizeof(struct db_password *));
+	bitmap_size = password_hash_sizes[salt->hash_size];
+	{
+		size_t size = (bitmap_size +
+		    sizeof(*salt->bitmap) * 8 - 1) /
+		    (sizeof(*salt->bitmap) * 8) * sizeof(*salt->bitmap);
+		salt->bitmap = mem_alloc_tiny(size, sizeof(*salt->bitmap));
+		memset(salt->bitmap, 0, size);
+	}
+
+	hash_size = bitmap_size >> PASSWORD_HASH_SHR;
+	if (hash_size > 1) {
+		size_t size = hash_size * sizeof(struct db_password *);
+		salt->hash = mem_alloc_tiny(size, MEM_ALIGN_WORD);
+		memset(salt->hash, 0, size);
+	}
 
 	salt->index = db->format->methods.get_hash[salt->hash_size];
 
@@ -723,8 +805,14 @@ static void ldr_init_hash_for_salt(struct db_main *db, struct db_salt *salt)
 	if ((current = salt->list))
 	do {
 		hash = hash_func(current->binary);
-		current->next_hash = salt->hash[hash];
-		salt->hash[hash] = current;
+		salt->bitmap[hash / (sizeof(*salt->bitmap) * 8)] |=
+		    1U << (hash % (sizeof(*salt->bitmap) * 8));
+		if (hash_size > 1) {
+			hash >>= PASSWORD_HASH_SHR;
+			current->next_hash = salt->hash[hash];
+			salt->hash[hash] = current;
+		} else
+			current->next_hash = current->next;
 		salt->count++;
 	} while ((current = current->next));
 }
@@ -746,13 +834,7 @@ static void ldr_init_hash(struct db_main *db)
  * hash table) vs. the complexity of DES_bs_cmp_all() for all computed hashes
  * at once (but calling it for each loaded hash individually).
  */
-		threshold = db->format->params.min_keys_per_crypt * 5;
-#if DES_BS_VECTOR
-		threshold /= ARCH_BITS_LOG * DES_BS_VECTOR;
-#else
-		threshold /= ARCH_BITS_LOG;
-#endif
-		threshold++;
+		threshold = 5 * ARCH_BITS / ARCH_BITS_LOG + 1;
 	}
 
 	if ((current = db->salts))
@@ -789,63 +871,6 @@ void ldr_fix_database(struct db_main *db)
 	db->loaded = 1;
 }
 
-/*
- * ldr_remove_salt() is called by ldr_remove_hash() when it happens to remove
- * the last password hash for a salt.
- */
-static void ldr_remove_salt(struct db_main *db, struct db_salt *salt)
-{
-	struct db_salt **current;
-
-	db->salt_count--;
-
-	current = &db->salts;
-	while (*current != salt)
-		current = &(*current)->next;
-	*current = salt->next;
-}
-
-void ldr_remove_hash(struct db_main *db, struct db_salt *salt,
-	struct db_password *pw)
-{
-	struct db_password **current;
-	int hash;
-
-	db->password_count--;
-
-	if (!--salt->count) {
-		salt->list = NULL; /* "single crack" mode might care */
-		ldr_remove_salt(db, salt);
-		return;
-	}
-
-/*
- * If there's no hash table for this salt, assume that next_hash fields are
- * unused and don't need to be updated.  Only bother with the list.
- */
-	if (salt->hash_size < 0) {
-		current = &salt->list;
-		while (*current != pw)
-			current = &(*current)->next;
-		*current = pw->next;
-		pw->binary = NULL;
-		return;
-	}
-
-	hash = db->format->methods.binary_hash[salt->hash_size](pw->binary);
-	current = &salt->hash[hash];
-	while (*current != pw)
-		current = &(*current)->next_hash;
-	*current = pw->next_hash;
-
-/*
- * If there's a hash table for this salt, assume that the list is only used by
- * "single crack" mode, so mark the entry for removal by "single crack" mode
- * code in case that's what we're running, instead of traversing the list here.
- */
-	pw->binary = NULL;
-}
-
 static int ldr_cracked_hash(char *ciphertext)
 {
 	unsigned int hash = 0;
@@ -877,7 +902,7 @@ static void ldr_show_pot_line(struct db_main *db, char *line)
 	if (line) {
 /* If just one format was forced on the command line, insist on it */
 		if (!fmt_list->next &&
-		    !fmt_list->methods.valid(ciphertext))
+		    !fmt_list->methods.valid(ciphertext, fmt_list))
 			return;
 
 		pos = line;
@@ -905,7 +930,13 @@ static void ldr_show_pot_line(struct db_main *db, char *line)
 
 void ldr_show_pot_file(struct db_main *db, char *name)
 {
+#ifdef HAVE_CRYPT
+	ldr_in_pot = 1;
+#endif
 	read_file(db, name, RF_ALLOW_MISSING, ldr_show_pot_line);
+#ifdef HAVE_CRYPT
+	ldr_in_pot = 0;
+#endif
 }
 
 static void ldr_show_pw_line(struct db_main *db, char *line)
@@ -913,7 +944,7 @@ static void ldr_show_pw_line(struct db_main *db, char *line)
 	int show;
 	char source[LINE_BUFFER_SIZE];
 	struct fmt_main *format;
-	char *(*split)(char *ciphertext, int index);
+	char *(*split)(char *ciphertext, int index, struct fmt_main *self);
 	int index, count, unify;
 	char *login, *ciphertext, *gecos, *home;
 	char *piece;
@@ -948,7 +979,7 @@ static void ldr_show_pw_line(struct db_main *db, char *line)
 	} else
 	for (found = pass = 0; pass == 0 || (pass == 1 && found); pass++)
 	for (index = 0; index < count; index++) {
-		piece = split(ciphertext, index);
+		piece = split(ciphertext, index, format);
 		if (unify)
 			piece = strcpy(mem_alloc(strlen(piece) + 1), piece);
 
@@ -956,15 +987,16 @@ static void ldr_show_pw_line(struct db_main *db, char *line)
 
 		if ((current = db->cracked_hash[hash]))
 		do {
-			if (!strcmp(current->ciphertext, piece))
+			char *pot = current->ciphertext;
+			if (!strcmp(pot, piece))
 				break;
 /* This extra check, along with ldr_cracked_hash() being case-insensitive,
  * is only needed for matching some pot file records produced by older
  * versions of John and contributed patches where split() didn't unify the
  * case of hex-encoded hashes. */
 			if (unify &&
-			    format->methods.valid(current->ciphertext) == 1 &&
-			    !strcmp(split(current->ciphertext, 0), piece))
+			    format->methods.valid(pot, format) == 1 &&
+			    !strcmp(split(pot, 0, format), piece))
 				break;
 		} while ((current = current->next));
 
