@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
  * Copyright (c) 1996-2001 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -22,7 +27,6 @@ extern void add64to64(int64 *dst, int64 *src);
 extern void neg64(int64 *dst);
 extern void mul32by32(int64 *dst, unsigned int m1, unsigned int m2);
 extern void mul64by32(int64 *dst, unsigned int m);
-extern void pow64of32(int64 *dst, unsigned int x, int n);
 extern unsigned int div64by32lo(int64 *src, unsigned int d);
 extern void div64by32(int64 *dst, unsigned int d);
 
