@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
  * Copyright (c) 1996-2000,2005,2008,2010 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -20,9 +25,6 @@
 #define ARCH_ALLOWS_UNALIGNED		0
 #define ARCH_INDEX(x)			((unsigned int)(unsigned char)(x))
 
-#define OS_TIMER			1
-#define OS_FLOCK			1
-
 #define CPU_DETECT			0
 
 #define DES_ASM				0
@@ -38,7 +40,7 @@
 
 #if 1
 #define DES_BS_VECTOR			4
-#define DES_BS_ALGORITHM_NAME		"128/128 BS AltiVec"
+#define DES_BS_ALGORITHM_NAME		"DES 128/128 AltiVec"
 #elif 0
 /* It is likely unreasonable to use S-box expressions requiring vsel when this
  * operation is only available in one of the two instruction sets.
@@ -47,10 +49,10 @@
 #define DES_BS				1
 #define DES_BS_VECTOR			5
 #define DES_BS_VECTOR_SIZE		8
-#define DES_BS_ALGORITHM_NAME		"128/128 BS AltiVec + 32/32 BS"
+#define DES_BS_ALGORITHM_NAME		"DES 128/128 AltiVec + 32/32"
 #else
 #define DES_BS_VECTOR			8
-#define DES_BS_ALGORITHM_NAME		"128/128 X2 BS AltiVec"
+#define DES_BS_ALGORITHM_NAME		"DES 128/128 X2 AltiVec"
 #endif
 
 #define MD5_ASM				0
