@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2000,2005 by Solar Designer
+ * Copyright (c) 1996-2000,2005,2008,2010 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -20,9 +25,6 @@
 #define ARCH_ALLOWS_UNALIGNED		0
 #define ARCH_INDEX(x)			((unsigned int)(unsigned char)(x))
 
-#define OS_TIMER			1
-#define OS_FLOCK			1
-
 #define CPU_DETECT			0
 
 #define DES_ASM				0
@@ -32,11 +34,26 @@
 #define DES_SCALE			0
 #define DES_EXTB			1
 #define DES_COPY			0
-#define DES_BS_ASM			1
-#define DES_BS				1
-#define DES_BS_VECTOR			2
+#define DES_BS_ASM			0
+#define DES_BS				3
 #define DES_BS_EXPAND			0
-#define DES_BS_ALGORITHM_NAME		"128/128 BS AltiVec"
+
+#if 1
+#define DES_BS_VECTOR			2
+#define DES_BS_ALGORITHM_NAME		"DES 128/128 AltiVec"
+#elif 0
+/* It is likely unreasonable to use S-box expressions requiring vsel when this
+ * operation is only available in one of the two instruction sets.
+ * So let's revert to less demanding S-box expressions. */
+#undef DES_BS
+#define DES_BS				1
+#define DES_BS_VECTOR			3
+#define DES_BS_VECTOR_SIZE		4
+#define DES_BS_ALGORITHM_NAME		"DES 128/128 AltiVec + 64/64"
+#else
+#define DES_BS_VECTOR			4
+#define DES_BS_ALGORITHM_NAME		"DES 128/128 X2 AltiVec"
+#endif
 
 #define MD5_ASM				0
 #define MD5_X2				1
@@ -44,5 +61,6 @@
 
 #define BF_ASM				0
 #define BF_SCALE			0
+#define BF_X2				0
 
 #endif
