@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2003,2006,2010 by Solar Designer
+ * Copyright (c) 1996-2003,2006,2010,2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 #define _XOPEN_SOURCE 500 /* for setitimer(2) and siginterrupt(3) */
@@ -10,6 +15,10 @@
 #define _POSIX_SOURCE
 #endif
 
+#define NEED_OS_TIMER
+#define NEED_OS_FORK
+#include "os.h"
+
 #ifdef _SCO_C_DIALECT
 #include <limits.h>
 #endif
@@ -33,8 +42,10 @@
 #include "misc.h"
 #include "params.h"
 #include "tty.h"
+#include "options.h"
 #include "config.h"
 #include "bench.h"
+#include "john.h"
 
 volatile int event_pending = 0;
 volatile int event_abort = 0, event_save = 0, event_status = 0;
@@ -123,11 +134,13 @@ void check_abort(int be_async_signal_safe)
 	tty_done();
 
 	if (be_async_signal_safe) {
-		write_loop(2, "Session aborted\n", 16);
+		if (john_main_process)
+			write_loop(2, "Session aborted\n", 16);
 		_exit(1);
 	}
 
-	fprintf(stderr, "Session aborted\n");
+	if (john_main_process)
+		fprintf(stderr, "Session aborted\n");
 	error();
 }
 
@@ -192,24 +205,14 @@ static void sig_remove_abort(void)
 #endif
 }
 
-#ifdef __CYGWIN32__
-
-static int sig_getchar(void)
+#if OS_FORK
+static void signal_children(int signum)
 {
-	int c;
-
-	if ((c = tty_getchar()) == 3) {
-		sig_handle_abort(CTRL_C_EVENT);
-		return -1;
-	}
-
-	return c;
+	int i;
+	for (i = 0; i < john_child_count; i++)
+		if (john_child_pids[i])
+			kill(john_child_pids[i], signum);
 }
-
-#else
-
-#define sig_getchar tty_getchar
-
 #endif
 
 static void sig_install_timer(void);
@@ -230,10 +233,29 @@ static void sig_handle_timer(int signum)
 		event_ticksafety = event_pending = 1;
 	}
 
-	if (sig_getchar() >= 0) {
-		while (sig_getchar() >= 0);
+	if (john_main_process) {
+		int c;
+#if OS_FORK
+		int new_abort = 0, new_status = 0;
+#endif
+		while ((c = tty_getchar()) >= 0) {
+			if (c == 3 || c == 'q') {
+#if OS_FORK
+				new_abort = 1;
+#endif
+				sig_handle_abort(0);
+			} else {
+#if OS_FORK
+				new_status = 1;
+#endif
+				event_status = event_pending = 1;
+			}
+		}
 
-		event_status = event_pending = 1;
+#if OS_FORK
+		if (new_abort || new_status)
+			signal_children(new_abort ? SIGTERM : SIGUSR2);
+#endif
 	}
 
 #if !OS_TIMER
@@ -245,6 +267,23 @@ static void sig_handle_timer(int signum)
 	errno = saved_errno;
 }
 
+#if OS_TIMER
+static void sig_init_timer(void)
+{
+	struct itimerval it;
+
+	it.it_value.tv_sec = TIMER_INTERVAL;
+	it.it_value.tv_usec = 0;
+#if defined(SA_RESTART) || defined(__DJGPP__)
+	it.it_interval = it.it_value;
+#else
+	memset(&it.it_interval, 0, sizeof(it.it_interval));
+#endif
+	if (setitimer(ITIMER_REAL, &it, NULL))
+		pexit("setitimer");
+}
+#endif
+
 static void sig_install_timer(void)
 {
 #if !OS_TIMER
@@ -252,7 +291,6 @@ static void sig_install_timer(void)
 	sig_timer_emu_init(TIMER_INTERVAL * clk_tck);
 #else
 	struct sigaction sa;
-	struct itimerval it;
 
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = sig_handle_timer;
@@ -264,14 +302,7 @@ static void sig_install_timer(void)
 	siginterrupt(SIGALRM, 0);
 #endif
 
-	it.it_value.tv_sec = TIMER_INTERVAL;
-	it.it_value.tv_usec = 0;
-#if defined(SA_RESTART) || defined(__DJGPP__)
-	it.it_interval = it.it_value;
-#else
-	memset(&it.it_interval, 0, sizeof(it.it_interval));
-#endif
-	if (setitimer(ITIMER_REAL, &it, NULL)) pexit("setitimer");
+	sig_init_timer();
 #endif
 }
 
@@ -287,6 +318,14 @@ static void sig_remove_timer(void)
 	signal(SIGALRM, SIG_DFL);
 }
 
+#if OS_FORK
+static void sig_handle_status(int signum)
+{
+	event_status = event_pending = 1;
+	signal(SIGUSR2, sig_handle_status);
+}
+#endif
+
 static void sig_done(void);
 
 void sig_init(void)
@@ -312,6 +351,16 @@ void sig_init(void)
 	sig_install_update();
 	sig_install_abort();
 	sig_install_timer();
+#if OS_FORK
+	signal(SIGUSR2, sig_handle_status);
+#endif
+}
+
+void sig_init_child(void)
+{
+#if OS_TIMER
+	sig_init_timer();
+#endif
 }
 
 static void sig_done(void)
