@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001,2006 by Solar Designer
+ * Copyright (c) 1996-2001,2006,2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -40,6 +45,12 @@ extern void sig_timer_emu_tick(void);
  */
 extern void sig_init(void);
 
+/*
+ * Performs additional (re-)initialization after fork().  Assumes that
+ * sig_init() has already been called.
+ */
+extern void sig_init_child(void);
+
 /*
  * Terminates the process if event_abort is set.
  */
