@@ -1,6 +1,6 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-99,2003,2004,2006,2010 by Solar Designer
+ * Copyright (c) 1996-99,2003,2004,2006,2010,2012,2013 by Solar Designer
  */
 
 #include <stdio.h>
@@ -8,16 +8,19 @@
 
 #include "misc.h"
 #include "params.h"
+#include "common.h"
 #include "memory.h"
 #include "signals.h"
 #include "loader.h"
 #include "logger.h"
 #include "status.h"
 #include "recovery.h"
+#include "options.h"
 #include "rpp.h"
 #include "rules.h"
 #include "external.h"
 #include "cracker.h"
+#include "john.h"
 
 static int progress = 0;
 static int rec_rule;
@@ -68,8 +71,9 @@ static void single_alloc_keys(struct db_keys **keys)
 		(*keys)->hash = mem_alloc_tiny(hash_size, MEM_ALIGN_WORD);
 	}
 
-	(*keys)->count = 0;
+	(*keys)->count = (*keys)->count_from_guesses = 0;
 	(*keys)->ptr = (*keys)->buffer;
+	(*keys)->have_words = 1; /* assume yes; we'll see for real later */
 	(*keys)->rule = rule_number;
 	(*keys)->lock = 0;
 	memset((*keys)->hash, -1, hash_size);
@@ -85,12 +89,23 @@ static void single_init(void)
 
 	length = single_db->format->params.plaintext_length;
 	key_count = single_db->format->params.min_keys_per_crypt;
-	if (key_count < SINGLE_HASH_MIN) key_count = SINGLE_HASH_MIN;
+	if (key_count < SINGLE_HASH_MIN)
+		key_count = SINGLE_HASH_MIN;
+/*
+ * We use "short" for buffered key indices and "unsigned short" for buffered
+ * key offsets - make sure these don't overflow.
+ */
+	if (key_count > 0x8000)
+		key_count = 0x8000;
+	while (key_count > 0xffff / length + 1)
+		key_count >>= 1;
 
 	if (rpp_init(rule_ctx, SUBSECTION_SINGLE)) {
 		log_event("! No \"single crack\" mode rules found");
-		fprintf(stderr, "No \"single crack\" mode rules found in %s\n",
-			cfg_name);
+		if (john_main_process)
+			fprintf(stderr,
+			    "No \"single crack\" mode rules found in %s\n",
+			    cfg_name);
 		error();
 	}
 
@@ -123,7 +138,7 @@ static void single_init(void)
 	crk_init(single_db, NULL, guessed_keys);
 }
 
-static int single_key_hash(char *key)
+static MAYBE_INLINE int single_key_hash(char *key)
 {
 	unsigned int hash, extra, pos;
 
@@ -164,8 +179,11 @@ out:
 	return hash;
 }
 
-static int single_add_key(struct db_keys *keys, char *key)
+static int single_process_buffer(struct db_salt *salt);
+
+static int single_add_key(struct db_salt *salt, char *key, int is_from_guesses)
 {
+	struct db_keys *keys = salt->keys;
 	int index, new_hash, reuse_hash;
 	struct db_keys_hash_entry *entry;
 
@@ -203,7 +221,12 @@ static int single_add_key(struct db_keys *keys, char *key)
 	strnfcpy(keys->ptr, key, length);
 	keys->ptr += length;
 
-	return ++(keys->count) >= key_count;
+	keys->count_from_guesses += is_from_guesses;
+
+	if (++(keys->count) >= key_count)
+		return single_process_buffer(salt);
+
+	return 0;
 }
 
 static int single_process_buffer(struct db_salt *salt)
@@ -212,7 +235,8 @@ static int single_process_buffer(struct db_salt *salt)
 	struct db_keys *keys;
 	size_t size;
 
-	if (crk_process_salt(salt)) return 1;
+	if (crk_process_salt(salt))
+		return 1;
 
 /*
  * Flush the keys list (since we've just processed the keys), but not the hash
@@ -227,7 +251,7 @@ static int single_process_buffer(struct db_salt *salt)
  * password hash computations vs. the "overhead".
  */
 	keys = salt->keys;
-	keys->count = 0;
+	keys->count = keys->count_from_guesses = 0;
 	keys->ptr = keys->buffer;
 	keys->lock++;
 
@@ -240,11 +264,11 @@ static int single_process_buffer(struct db_salt *salt)
 		do {
 			current = single_db->salts;
 			do {
-				if (current == salt) continue;
-				if (!current->list) continue;
+				if (current == salt || !current->list)
+					continue;
 
-				if (single_add_key(current->keys, keys->ptr))
-				if (single_process_buffer(current)) return 1;
+				if (single_add_key(current, keys->ptr, 1))
+					return 1;
 			} while ((current = current->next));
 			keys->ptr += length;
 		} while (--keys->count);
@@ -254,7 +278,8 @@ static int single_process_buffer(struct db_salt *salt)
 
 	keys = salt->keys;
 	keys->lock--;
-	if (!keys->count && !keys->lock) keys->rule = rule_number;
+	if (!keys->count && !keys->lock)
+		keys->rule = rule_number;
 
 	return 0;
 }
@@ -262,7 +287,6 @@ static int single_process_buffer(struct db_salt *salt)
 static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 	char *rule)
 {
-	struct db_keys *keys;
 	struct list_entry *first, *second;
 	int first_number, second_number;
 	char pair[RULE_WORD_SIZE];
@@ -273,21 +297,23 @@ static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 	if (!(first = pw->words->head))
 		return -1;
 
-	keys = salt->keys;
-
 	first_number = 0;
 	do {
 		if ((key = rules_apply(first->data, rule, 0, NULL)))
 		if (ext_filter(key))
-		if (single_add_key(keys, key))
-		if (single_process_buffer(salt)) return 1;
-		if (!salt->list) return 2;
-		if (!pw->binary) return 0;
+		if (single_add_key(salt, key, 0))
+			return 1;
+		if (!salt->list)
+			return 2;
+		if (!pw->binary)
+			return 0;
 
-		if (++first_number > SINGLE_WORDS_PAIR_MAX) continue;
+		if (++first_number > SINGLE_WORDS_PAIR_MAX)
+			continue;
 
 		c = (unsigned int)first->data[0] | 0x20;
-		if (c < 'a' || c > 'z') continue;
+		if (c < 'a' || c > 'z')
+			continue;
 
 		second_number = 0;
 		second = pw->words->head;
@@ -300,10 +326,12 @@ static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 
 				if ((key = rules_apply(pair, rule, split, NULL)))
 				if (ext_filter(key))
-				if (single_add_key(keys, key))
-				if (single_process_buffer(salt)) return 1;
-				if (!salt->list) return 2;
-				if (!pw->binary) return 0;
+				if (single_add_key(salt, key, 0))
+					return 1;
+				if (!salt->list)
+					return 2;
+				if (!pw->binary)
+					return 0;
 			}
 
 			if (first->data[1]) {
@@ -313,13 +341,15 @@ static int single_process_pw(struct db_salt *salt, struct db_password *pw,
 
 				if ((key = rules_apply(pair, rule, 1, NULL)))
 				if (ext_filter(key))
-				if (single_add_key(keys, key))
-				if (single_process_buffer(salt)) return 1;
-				if (!salt->list) return 2;
-				if (!pw->binary) return 0;
+				if (single_add_key(salt, key, 0))
+					return 1;
+				if (!salt->list)
+					return 2;
+				if (!pw->binary)
+					return 0;
 			}
 		} while (++second_number <= SINGLE_WORDS_PAIR_MAX &&
-			(second = second->next));
+		    (second = second->next));
 	} while ((first = first->next));
 
 	return 0;
@@ -332,8 +362,9 @@ static int single_process_salt(struct db_salt *salt, char *rule)
 	int status, have_words = 0;
 
 	keys = salt->keys;
-	if (!keys)
-		return 0;
+
+	if (!keys->have_words)
+		goto no_own_words;
 
 	last = &salt->list;
 	pw = *last;
@@ -360,13 +391,17 @@ next:
 	} while ((pw = pw->next));
 
 	if (keys->count && rule_number - keys->rule > (key_count << 1))
-		if (single_process_buffer(salt)) return 1;
+		if (single_process_buffer(salt))
+			return 1;
 
-	if (!keys->count) keys->rule = rule_number;
+	if (!keys->count)
+		keys->rule = rule_number;
 
 	if (!have_words) {
-		if (keys->count && single_process_buffer(salt)) return 1;
-		salt->keys = NULL;
+		keys->have_words = 0;
+no_own_words:
+		if (keys->count && single_process_buffer(salt))
+			return 1;
 	}
 
 	return 0;
@@ -381,6 +416,15 @@ static void single_run(void)
 
 	saved_min = rec_rule;
 	while ((prerule = rpp_next(rule_ctx))) {
+		if (options.node_count) {
+			int for_node = rule_number % options.node_count + 1;
+			if (for_node < options.node_min ||
+			    for_node > options.node_max) {
+				rule_number++;
+				continue;
+			}
+		}
+
 		if (!(rule = rules_reject(prerule, 0, NULL, single_db))) {
 			log_event("- Rule #%d: '%.100s' rejected",
 				++rule_number, prerule);
@@ -388,7 +432,7 @@ static void single_run(void)
 		}
 
 		if (strcmp(prerule, rule))
-			log_event("- Rule #%d: '%.100s' accepted as '%s'",
+			log_event("- Rule #%d: '%.100s' accepted as '%.100s'",
 				rule_number + 1, prerule, rule);
 		else
 			log_event("- Rule #%d: '%.100s' accepted",
@@ -406,9 +450,12 @@ static void single_run(void)
 
 		salt = single_db->salts;
 		do {
-			if (!salt->list) continue;
-			if (single_process_salt(salt, rule)) return;
-			if (!salt->keys) continue;
+			if (!salt->list)
+				continue;
+			if (single_process_salt(salt, rule))
+				return;
+			if (!salt->keys->have_words)
+				continue;
 			have_words = 1;
 			if (salt->keys->rule < min)
 				min = salt->keys->rule;
@@ -417,7 +464,8 @@ static void single_run(void)
 		rec_rule = min;
 		rule_number++;
 
-		if (have_words) continue;
+		if (have_words)
+			continue;
 
 		log_event("- No information to base%s candidate passwords on",
 			rule_number > 1 ? " further" : "");
@@ -435,9 +483,11 @@ static void single_done(void)
 				"candidate passwords, if any");
 
 			do {
-				if (!salt->list || !salt->keys) continue;
+				if (!salt->list)
+					continue;
 				if (salt->keys->count)
-				if (single_process_buffer(salt)) break;
+					if (single_process_buffer(salt))
+						break;
 			} while ((salt = salt->next));
 		}
 
