@@ -1,6 +1,11 @@
 /*
  * This file is part of John the Ripper password cracker,
- * Copyright (c) 1996-2001,2006 by Solar Designer
+ * Copyright (c) 1996-2001,2006,2011,2013 by Solar Designer
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted.
+ *
+ * There's ABSOLUTELY NO WARRANTY, express or implied.
  */
 
 /*
@@ -20,7 +25,9 @@
 struct status_main {
 	clock_t start_time;
 	unsigned int guess_count;
-	int64 crypts;
+	int64 combs, crypts, cands;
+	unsigned int combs_ehi;
+	int compat;
 	int pass;
 	int progress;
 };
@@ -51,9 +58,17 @@ extern void status_init(int (*get_progress)(void), int start);
 extern void status_ticks_overflow_safety(void);
 
 /*
- * Updates the crypts count.
+ * Updates the combinations and crypts counters by adding the supplied numbers
+ * to them.
+ * Calls status_ticks_overflow_safety() once in a while.
  */
-extern void status_update_crypts(unsigned int count);
+extern void status_update_crypts(int64 *combs, unsigned int crypts);
+
+/*
+ * Updates the candidates counter by adding the supplied number to it.
+ * Calls status_ticks_overflow_safety() once in a while.
+ */
+extern void status_update_cands(unsigned int cands);
 
 /*
  * Returns the elapsed time in seconds.
